<?php
/*
  This code is part of GOsa (https://gosa.gonicus.de)
  Copyright (C) 2003  Cajus Pollmeier

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* Configuration file location */
define ("CONFIG_DIR", "/etc/gosa");

/* Include required files */
require_once ("ldap.inc");
require_once ("config.inc");
require_once ("userinfo.inc");
require_once ("plugin.inc");
require_once ("pluglist.inc");
require_once ("tabs.inc");
require_once ("debuglib.inc");


/* Define constants for debugging */
define ("DEBUG_TRACE",   1);
define ("DEBUG_LDAP",    2);
define ("DEBUG_MYSQL",   4);
define ("DEBUG_SHELL",   8);
define ("DEBUG_POST",   16);
define ("DEBUG_SESSION",32);
define ("DEBUG_CONFIG", 64);


/* Rewrite german 'umlauts' to get better results */
$REWRITE= array( "" => "ae",
                 "" => "oe",
                 "" => "ue",
                 "" => "ss" );


/* Function to include all class_ files starting at a
   given directory base */
function get_dir_list($folder= ".")
{
	$currdir=getcwd();
	if ($folder){
		@chdir("$folder");
	}

	$dh = opendir(".");
	while(false !== ($file = readdir($dh))){

    		if (preg_match("/^class_/", $file)) {
			require_once($file);
		}

	        /* insert all directories in an array */
	        if(is_dir($file) && $file!="." && $file!=".." && $file!="CVS"){
			get_dir_list($file);
		}
	}

	closedir($dh);
	chdir($currdir);
}


/* Debug level action */
function DEBUG($level, $line, $function, $file, $data, $info="")
{
	if ($_SESSION['DEBUGLEVEL'] & $level){
		$output= "DEBUG[$level] ";
		if ($function != ""){
			$output.= "($file:$function():$line) - $info: ";
		} else {
			$output.= "($file:$line) - $info: ";
		}
		echo $output;
		if (is_array($data)){
			print_a($data);
		} else {
			echo "'$data'";
		}
		echo "<br>";
	}
}


/* Simple function to get browser language and convert it to
   xx_XY needed by locales. Ignores sublanguages and weights. */
function get_browser_language()
{
	/* Get list of languages */
	if (isset($_SERVER['HTTP_ACCEPT_LANGUAGE'])){
		$lang= preg_replace("/\s+/", "", $_SERVER['HTTP_ACCEPT_LANGUAGE']);
		$languages= split (',', $lang);
	} else {
		$languages= array("C");
	}

	/* Walk through languages and get first supported */
	foreach ($languages as $val){

		/* Strip off weight */
		$lang= preg_replace("/;q=.*$/i", "", $val);

		/* Simplify sub language handling */
		$lang= preg_replace("/-.*$/", "", $lang);

		return (strtolower($lang)."_".strtoupper($lang));
	}
}


/* Rewrite ui object to another dn */
function change_ui_dn($dn, $newdn)
{
	$ui= $_SESSION['ui'];
	if ($ui->dn == $dn){
		$ui->dn= $newdn;
		$_SESSION['ui']= $ui;
	}
}


/* Return theme path for specified file */
function get_template_path($filename= '', $plugin= FALSE)
{
	global $config, $BASE_DIR;

	if (!isset($config->data['MAIN']['THEME'])){
		$theme= 'default';
	} else {
		$theme= $config->data['MAIN']['THEME'];
	}

	/* Return path for empty filename */
	if ($filename == ''){
		return ("themes/$theme/");
	}
	
	/* Return plugin dir or root directory? */
	if ($plugin){
		$nf= preg_replace("!^".$BASE_DIR."/!", "", $_SESSION['plugin_dir']);
		if (file_exists("$BASE_DIR/ihtml/themes/$theme/$nf")){
			return ("$BASE_DIR/ihtml/themes/$theme/$nf");
		}
		if (file_exists("$BASE_DIR/ihtml/themes/default/$nf")){
			return ("$BASE_DIR/ihtml/themes/default/$nf");
		}
		return ($_SESSION['plugin_dir']."/$filename");
	} else {
		if (file_exists("themes/$theme/$filename")){
			return ("themes/$theme/$filename");
		}
		if (file_exists("$BASE_DIR/ihtml/themes/$theme/$filename")){
			return ("$BASE_DIR/ihtml/themes/$theme/$filename");
		}
		if (file_exists("themes/default/$filename")){
			return ("themes/default/$filename");
		}
		if (file_exists("$BASE_DIR/ihtml/themes/default/$filename")){
			return ("$BASE_DIR/ihtml/themes/default/$filename");
		}
		return ($filename);
	}
}


function array_remove_entry($entry, $stack)
{
	$temp= array();
	foreach ($stack as $key => $value){
		if ($value != $entry){
			$temp[$key]= $value;
		}
	}
	return($temp);
}

## array_strip ($toberemoved, $data)
##
function array_strip($toberemoved, $data)
{
    $tmp= array();
    
    foreach ($data as $needle){
      if (!in_array($needle, $toberemoved)){
        $tmp[]= $needle;
      }
    }

    return ($tmp);
}


## printe (function, message)
##
function printe ($function, $message)
{
  echo ("Error in function $function<br>\n");
  echo ("Function returned: '$message'<br>\n");

  gosa_log ("Error in function $function: $message");
  
  die ();
}


## log (message)
##
function gosa_log ($message)
{
  global $ui;
 
  $username= "";

  if (isset($ui)){
    if ($ui->username != ""){
      $username= "[$ui->username]";
    }
  }
  
  syslog(LOG_WARNING,"GOsa$username: $message");
}


## ldap_init (server, base [, binddn, pass])
##
function ldap_init ($server, $base, $binddn='', $pass='')
{
  $ldap = new LDAP ($binddn, $pass, $server);

  if (!(strpos($ldap->error, "Could not bind to") === false)) {
    return (0);
  }

  if ($ldap->error != "Success"){
    printe ("ldap_init ()", 
            $ldap->error);
  }

  $ldap->cd ($base);

  return $ldap;
}


## ldap_login_user (username, password)
##
function ldap_login_user ($username, $password)
{
  /* Include global configuration */
  global $config;

  /* look through the entire ldap */
  $tldap = new LDAP("","",$config->current['SERVER'], "true");
  if ($tldap->error != "Success"){
	printe ("ldap_login_user()", $tldap->error);
  }
  $tldap->cd($config->current['BASE']);
  $tldap->search("(&(uid=$username)(objectClass=gosaAccount))");
 
  /* get results, only a count of 1 is valid */
  switch ($tldap->count()){
  
	/* user not found */
	case 0:	return (0);
		break;

	/* valid uniq user */
	case 1: 
		break;

	/* found more than one matching id */
	default:
		printe ("ldap_login_user()","Username is not uniq. Check your ldap database.");
  }

  /* got user dn, fill acl's */
  $tldap->fetch();
  $ui= new userinfo($config, $tldap->getDN());
  $ui->username= $username;
  $ui->password= $password;

  /* password check, bind as user with supplied password  */
  $tldap = new LDAP($ui->dn, $password, $config->current['SERVER'], "true");
  if ($tldap->error != "Success"){
	return (0);
  }
 
  /* Username is set, load subtreeACL's now */
  $ui->loadACL();

  return ($ui);
}


## getmicrotime()
#
function getmicrotime(){ 
  list($usec, $sec) = explode(" ",microtime()); 
  return ((float)$usec + (float)$sec); 
}

## must(string)
#  like _(), but adds a red * to the end
function must($string)
{
  echo _($string);
  echo "<font class=\"must\">*</font>";
}

## add_lock (object, user)
##
function add_lock ($object, $user)
{
  global $config;

  if ($object == "" || $user == ""){
    printe ("add_lock()", "Parameters not set correctly");
  }

  $ldap= $config->get_ldap_link();
  $ldap->cd ($config->current['CONFIG']);
  $ldap->search("(&(objectClass=gosaLockEntry)(gosaUser=$user)(gosaObject=$object))");
  if ($ldap->error != "Success"){
    printe ("add_lock()", $ldap->error);
  }

  # Lock exists?
  if ($ldap->count() == 0){
    # create lock
    $attrs= array();
    $name= md5($object);
    $ldap->cd("cn=$name,".$config->current['CONFIG']);
    $attrs["objectClass"] = "gosaLockEntry";
    $attrs["gosaUser"] = $user;
    $attrs["gosaObject"] = $object;
    $attrs["cn"] = "$name";
    $ldap->add($attrs);
    if ($ldap->error != "Success"){
      printe ("add_lock()", $ldap->error);
    }

  }

  # success
  return (1);
}


## del_lock (object)
##
function del_lock ($object)
{
  global $config;

  if ($object == ""){
    return (0);
  }

  $ldap= $config->get_ldap_link();
  $ldap->cd ($config->current['CONFIG']);
  $ldap->search ("(&(objectClass=gosaLockEntry)(gosaObject=$object))");
  $attrs= $ldap->fetch();
  if ($ldap->getDN() != "" && $ldap->error == "Success"){
    $ldap->rmdir ($ldap->getDN());
  }
  
  if ($ldap->error != "Success"){
    return (0);
  }

  # Success
  return (1);
}


## user get_lock (object);
##
function get_lock ($object)
{
  global $config;
  
  if ($object == ""){
    printe ("get_lock()", "Parameters not set correctly");
  }

  $user= "";
  $ldap= $config->get_ldap_link();
  $ldap->cd ($config->current['CONFIG']);
  $ldap->search("(&(objectClass=gosaLockEntry)(gosaObject=$object))");
  if ($ldap->error != "Success"){
    printe ("get_lock()", $ldap->error);
  }

  # validity check
  if ($ldap->count() > 1){
    printe ("get_lock()", "Multiple locks for this object - should not be possible. Please check your LDAP.");
  } elseif ($ldap->count() == 1){
    $attrs = $ldap->fetch();
    $user= $attrs['gosaUser'][0];
  }

  return $user;
}

## array get_list (subtreeACL, filter, [base])
##
function get_list($subtreeACL, $filter, $base="")
{
  global $config;

  $ldap= $config->get_ldap_link();
  if ($base == ""){
    $ldap->cd ($config->current['BASE']);
  } else {
    $ldap->cd ($base);
  }
  $ldap->search ($filter);
  $result= array();

  while($attrs = $ldap->fetch()) {
    $dn= preg_replace("/[ ]*,[ ]*/", ",", $ldap->getDN());
    foreach ($subtreeACL as $key => $value){
      if (is_integer(strpos($dn, $key))){
        $attrs["dn"]= $dn;
        $result[]= $attrs;
        break;
      }
    }
  }
  return ($result);
}


## print_sort_ranges()
##
function print_sort_ranges($sortranges)
{
	echo get_sort_ranges($sortranges);
}

## get_sort_ranges()
##
function get_sort_ranges($sortranges)
{
  global $plug;

  $res= "";

  if (isset($plug)){
    $plugin= "plug=$plug&";
  } else {
    $plugin= "";
  }
  foreach ($sortranges as $key => $value){
    $res.= "[<a href=\"main.php?".$plugin."sorting=$key\">"._($key)."</a>]&nbsp;";
  }

  return $res;
}


## get_permissions ($userinfo)
##
function get_permissions ($userinfo, $subtreeACL)
{
  global $config;

  $base= $config->current['BASE'];
  $tmp= "d,".$userinfo;
  $sacl= array();

  # sort subacl's for lenght
  foreach ($subtreeACL as $key => $value){
    $sacl[$key]= strlen($key);
  }
  arsort ($sacl);
  reset ($sacl);

#  echo "Sorted acl's:<br>";
#  print_a ($sacl);
#  echo "<br>";

  while (is_integer(strpos($tmp,","))){
    $tmp= ltrim(strstr($tmp, ","), ",");
#    echo ": $tmp<br>";

    # check for acl that applies
    foreach ($sacl as $key => $value){
      if (is_integer(strpos($tmp, $key))){
#      	  echo "Returning: $subtreeACL[$key]<br>";
          return ($subtreeACL[$key]);
      }
    }
  }

  return "";
}


## get_module_permission($acl, $module)
##
function get_module_permission($acl, $module, $dn)
{
  global $ui;

  /* Check for selfflag */
  if (preg_match("/^!/", $acl)){
	if ($dn != "" && $dn != $ui->dn){
		return ("#none#");
	}
	$acl= preg_replace("/^!/", "", $acl);
  }
  $acl= preg_replace("/^:/", "", $acl);

  $tmp= split(",", $acl);
  foreach ($tmp as $mod){
    if (preg_match("/$module#/", $mod)){
      return (strstr($mod, "#")."#");
    }
    if (preg_match("/[^#]$module$/", $mod)){
      return ("#all#");
    }
    if (preg_match("/^all$/", $mod)){
      return ("#all#");
    }
  }

  return ("#none#");
}


function get_userinfo()
{
  global $ui;

  return $ui;
}


/* Yam, yam function */
function get_smarty()
{
  global $smarty;

  return $smarty;
}


## convert_department_dn($dn)
##
function convert_department_dn($dn)
{
  $dep= "";
  foreach (split (",", $dn) as $val){
    if (preg_match ("/ou=/", $val)){
      $dep= preg_replace("/ou=([^,]+)/", "\\1", $val)."/$dep";
    }
    if (preg_match ("/l=/", $val)){
      $dep= preg_replace("/l=([^,]+)/", "\\1", $val)."/$dep";
    }
  }
  return rtrim($dep, "/");
}


## get_departments
##
function get_departments()
{
  global $config;

  $result= array();

  $ldap= $config->get_ldap_link();
  $ldap->cd ($config->current['BASE']);
  $ldap->search ("(objectClass=gosaDepartment)");
  $result['/']= $config->current['BASE'];
  while ($attrs= $ldap->fetch()){
    $dn= $ldap->getDN();
    $result[utf8_decode(convert_department_dn($dn))]= $dn;
  }

  return ($result);
}


## chkacl($acl, $name)
##
function chkacl($acl, $name)
{

  # matches?
  if (preg_match("/#$name#/", $acl) || $acl == "#all#"){
    return ("");
  }

  # no restrictions
  return (" disabled ");
}


## is_phone_nr($nr)
##
function is_phone_nr($nr)
{
  if ($nr == ""){
    return (TRUE);
  }
  return preg_match ("/^[0-9 ()+*-]+$/", $nr);
}


## is_url($url)
##
function is_url($url)
{
  if ($url == ""){
    return (TRUE);
  }
  return preg_match ("/^(http|https):\/\/((?:[a-zA-Z0-9_-]+\.?)+):?(\d*)/", $url);
}


## is_dn($dn)
##
function is_dn($dn)
{
  if ($dn == ""){
    return (TRUE);
  }
  return preg_match ("/^[a-z0-9 _-]+$/i", $dn);
}

## is_uid($uid)
##
function is_uid($uid)
{
  if ($uid == ""){
    return (TRUE);
  }
  return preg_match ("/^[a-z0-9-]+$/", $uid);
}

## is_uid($id)
##
function is_id($id)
{
  if ($id == ""){
    return (FALSE);
  }
  return preg_match ("/^[0-9]+$/", $id);
}

## is_path($path)
##
function is_path($path)
{
  if ($path == ""){
    return (TRUE);
  }
  return preg_match ("/\/.+$/", $path);
}

## is_email($address)
##
function is_email($address)
{
  if ($address == ""){
    return (TRUE);
  }
  return preg_match ("/^[._a-z0-9%-]+@[_a-z0-9-]+(\.[a-z0-9]+)(\.[a-z0-9]+)*$/i", $address);
}

## is_dns_notation($input)
##
function is_dns_notation($input)
{
  if ($input == ""){
    return (TRUE);
  }
  return preg_match ("/^[a-z0-9.-]+\.$/i", $input);
}


## print_red($string)
##
function print_red (){
	/* Check number of arguments */
	if (func_num_args() < 1){
		return;
	}

	/* Get arguments, save string */
	$array = func_get_args();
	$string= $array[0];

	/* Step through arguments */
	for ($i= 1; $i<count($array); $i++){
		$string= preg_replace ("/%s/", $array[$i], $string, 1);
	}

	echo ("<div align=center style=\"border-width:3px; border-style:solid; ".
		"border-color:red; background-color:black;\"><br>".
		"<font color=#FF0000><b>$string</b></font><br><br></div>\n");
}


## gen_locked_message (user, dn)
##
function gen_locked_message($user, $dn)
{
  global $plug, $config;

  $_SESSION['dn']= $dn;
  $ldap= $config->get_ldap_link();
  $ldap->cat ($user);
  $attrs= $ldap->fetch();
  $uid= $attrs["uid"][0];
  require_once ("islocked.inc");
}


function gen_tabs($items, $active, $disabled= FALSE)
{
  global $plug;
  ?>
  <table cellpadding=0 cellspacing=0 border=0 style="width:100%;">
   <tr>
   <?php

     for ($i= 0; $i<count($items); $i++){
       if ($i < $active){
         $class= "tab_left";
       }
       if ($i > $active){
         $class= "tab_right";
       }
       if ($i == ($active+1)){
         $class= "tab_near_active";
       }
       if ($i == $active){
         $class= "tab_active";
       }

       ?>
       <td style="vertical-align:bottom; width:100px;">
       <?php
       echo "<div class=\"$class\"><input type=\"submit\" name=\"tab$i\"".
            " class=\"$class\" value=\""._($items[$i])."\"";
       if ($disabled){
         echo " disabled";
       }
       echo "></div></td>";
     }

   ?>
     <td style="vertical-align:bottom;">
       <div class="tab_border">&nbsp;</div>
     </td>
   </tr>
  </table>
  <?php
}


function to_string ($value)
{
  if (is_array($value)){
    $ret= "";
    foreach ($value as $line){
      $ret.= $line."<br>\n";
    }
    return ($ret);
  } else {
    return ($value);
  }
}

function minimum_version($vercheck)
{
  $minver = (int)str_replace('.', '', $vercheck);
  $curver = (int)str_replace('.', '', phpversion());

  if($curver >= $minver){
    return (true);
  }

  return (false);
}

function get_dn ($uid)
{
  global $config;

  $ldap= $config->get_ldap_link();
  $ldap->cd ($config->current['BASE']);
  $ldap->search ("(&(uid=$uid)(objectClass=person)".
                 "(objectClass=organizationalPerson)(objectClass=inetOrgPerson))");
  if ($ldap->count() != 1){
    printe ("plugin users", "uid is not unique");
  }
  $ldap->fetch ();
  return ($ldap->getDN());
}


function get_printer_list($cups_server)
{
  $res= array();
  if (function_exists('cups_get_dest_list')){
    $dest_list= cups_get_dest_list ($cups_server);

    foreach ($dest_list as $prt){
      $attr= cups_get_printer_attributes ($cups_server, $prt->name);

      foreach ($attr as $prt_info){
        if ($prt_info->name == "printer-info"){
          $info= $prt_info->value;
          break;
        }
      }
      $res[$prt->name]= "$info [$prt->name]";
    }
  }

  return $res;
}


/*
* sess_del
* Remove variables from session.
*/
function sess_del ($var)
{
  /* New style */
  unset ($_SESSION[$var]);

  /* ... work around, since the first one
     doesn't seem to work all the time */
  session_unregister ($var);
}


function show_errors($message)
{
	$complete= "";
	foreach ($message as $error){
		if ($complete == ""){
			$complete= $error;
		} else {
			$complete= "$error<br>$complete";
		}
	}
	print_red($complete);
}


function show_ldap_error($message)
{
	if ($message != "Success"){
		print_red (_("LDAP error:")." $message");
	}
}


function rewrite($s)
{
  global $REWRITE;

  foreach ($REWRITE as $key => $val){
    $s= preg_replace("/$key/i", "$val", $s);
  }

  return ($s);
}

function dn2base($dn)
{
	global $config;

	$tmp= preg_replace ('/^[^,]+,((ou=people)|(ou=groups)),/', '', $dn);
	$key= array_search ($tmp, $config->departments);
	if ($key === FALSE || $key === NULL){
		$key= "/";
	}

	return $key;
}


function change_password ($dn, $password, $mode=0, $hash= "")
{
  global $config;
  $newpass= "";

  # Read current password entry for $dn
  $ldap= $config->get_ldap_link();
  $ldap->cat ($dn);
  $attrs= $ldap->fetch ();

  # Preset encryption type if not set

  # Check for encryption type and do what's needed
  if ((isset($attrs['userPassword'][0]) && preg_match ("/^{([^}]+)}(.+)/", $attrs['userPassword'][0], $matches))
	|| $hash != ""){

    # Check for supported algorithm
    mt_srand((double) microtime()*1000000);

    if ($hash == ""){
      $hash= strtolower($matches[1]);
    }
    switch ($hash){
      case "crypt":
                $newpass= "{crypt}".crypt($password, substr(session_id(),0,2));
                break;

      case "md5":
                $newpass = "{md5}".base64_encode(pack("H*",md5($password)));
                break;

      case "ssha":
                $salt=mhash_keygen_s2k(MHASH_SHA1,$password,substr(pack("h*",md5(mt_rand())),0,8),4);
                $newpass= "{ssha}".base64_encode(mhash(MHASH_SHA1, $password.$salt).$salt);
                break;

      case "kerberos":
		$cfg= $config->data['SERVERS']['KERBEROS'];
		if (!function_exists('kadm5_init_with_password')){
			print_red (_("Warning: Can't set kerberos password. Your PHP version has no ".
					"kerberos support, password has not been changed."));
		} else {
	                $handle = kadm5_init_with_password($cfg['SERVER'],
	                                $cfg['REALM'], $cfg['ADMIN'], $cfg['PASSWORD']);
	                if ($handle === FALSE){
	                  print_red (_("Kerberos database communication failed"));
	                }
	                $ret= kadm5_chpass_principal($handle, $attrs['uid'][0]."@".$cfg['REALM'],
	                                $password);
	                if ($ret === FALSE){
	                  print_red (lang("Changing password in kerberos database failed"));
	                }
	                kadm5_destroy($handle);

	                $newpass= "{kerberos}".$attrs['uid'][0]."@".$cfg['REALM'];
		}
                break;

      default:  printe ("change_password", "Unsupported password hash detected. Can't ".
                                           "generate compatible password.");
                break;
    }

  } else {
    # Crypt it by default
    $newpass= "{crypt}".crypt($password, substr(session_id(),0,2));
  }

  # Update shadow timestamp?
  if (isset($attrs["shadowLastChange"][0])){
    $shadow= (int)(date("U") / 86400);
  } else {
    $shadow= 0;
  }

  # Write back modified entry
  $ldap->cd($dn);
  $attrs= array();
  if ($password == ""){
    $attrs['userPassword']= array();
  } else {
    $attrs['userPassword']= $newpass;
  }

  # Not for groups
  if ($mode == 0){
    if ($shadow != 0){
      $attrs['shadowLastChange']= $shadow;
    }

    # Create SMB Password
    $tmp= $config->data['MAIN']['SMBHASH']." $password";
    @DEBUG (DEBUG_LDAP, __LINE__, __FUNCTION__, __FILE__,
	$tmp, "Execute");

    $hash= shell_exec ($tmp);
    if ($hash == ""){
	print_red (_("Setting for SMBHASH in ".CONFIG_DIR."/gosa.conf is incorrect! Can't change Samba password."));
    } else {
      list($lm,$nt)= split (":", trim($hash));
      if ($config->current['SAMBAVERSION'] == 3){
        $attrs['sambaLMPassword']= $lm;
        $attrs['sambaNTPassword']= $nt;
      } else {
        $attrs['lmPassword']= $lm;
        $attrs['ntPassword']= $nt;
      }
    }
  }

  $ldap->modify($attrs);
  if ($ldap->error != 'Success'){
    printe ("set_password", $ldap->error);
  }
}


function gen_uid ($word, $addword)
{
  global $config;

  $passed= FALSE;
  $proposed= array();
  $uids= array();

  $ldap= $config->get_ldap_link();
  $ldap->cd($config->current['BASE']);
  $ldap->search ("(uid=*)");
  while ($attrs= $ldap->fetch()){
    $uids[]= $attrs['uid'][0];
  }

  # normalize words, strip umlauts, etc...
  $s1= preg_replace("/ /", "", strtolower ($word));
  $s2= preg_replace("/ /", "", strtolower ($addword));
  $s1= rewrite($s1);
  $s2= rewrite($s2);

  # short names
  if (strlen($s1) < 7 && !in_array($s1, $uids)){
    $proposed[]= $s1;
  }

  # generate combinations
  foreach (array ("4+1", "5+1", "4+2", "5+2", "3+2", "3+3", "2+2", "2+3", "2+5", "2+4", "1+5", "1+4") as $val){
    list($t1,$t2)= explode ("+", $val);
    $wrt= substr($s1, 0, $t1).substr($s2, 0, $t2);
    if (!in_array($wrt, $uids)){
      $proposed[]= $wrt;
    }
  }

  # reverse action
  if (strlen($s2) < 7 && !in_array($s2, $uids)){
    $proposed[]= $s2;
  }
  foreach (array ("4+1", "5+1", "4+2", "5+2", "3+2", "3+3", "2+2", "2+3", "2+5", "2+4", "1+5", "1+4") as $val){
    list($t1,$t2)= explode ("+", $val);

    $wrt= substr($s2, 0, $t1).substr($s1, 0, $t2);
    if (!in_array($wrt, $uids)){
      $proposed[]= $wrt;
    }
  }

  # Aahrg, just to be sure
  if (count($proposed) == 0){
    $num= 0;
    $prefix= substr($s1, 0, 2).substr($s2, 0, 2);
    while ($num++ < 9999){
      $wrt= sprintf("%s%4d", $prefix, $num);
      if (!in_array($wrt, $uids)){
        $proposed[]= $wrt;
        break;
      }
    }
  }

  return (array_unique($proposed));
}


function check_command($cmdline)
{
	$cmd= preg_replace("/ .*$/", "", $cmdline);

	/* Check if command exists in filesystem */
	if (!file_exists($cmd)){
		return (FALSE);
	}

	/* Check if command is executable */
	if (!is_executable($cmd)){
		return (FALSE);
	}

	return (TRUE);
}


function print_header($image, $headline)
{
	echo "<div class=\"plugtop\">\n";
	echo "  <img src=\"$image\" align=center>$headline\n";
	echo "</div>\n";
}

?>
