<?php
/*****************************************************************************
 newldap.inc - version 1.0
 Copyright (C) 2003 Alejandro Escanero Blanco alex@ofmin.com

 Based in code of ldap.inc of
 Copyright (C) 1998  Eric Kilfoil eric@ipass.net
*****************************************************************************/

class LDAP{

	var $hascon=false;
	var $hasres=false;
	var $basedn="";
	var $cid;
	var $error = ""; // Any error messages to be returned can be put here
	var $start = 0; // 0 if we are fetching the first entry, otherwise 1
	var $objectClasses = array(); // Information read from slapd.oc.conf

	function LDAP($binddn,$bindpw, $hostname)
	{
		error_reporting(0);
		$this->hascon=false;
		if ($this->cid=ldap_connect($hostname)) {
			ldap_set_option($this->cid, LDAP_OPT_PROTOCOL_VERSION, 3);

			$this->error = "No Error";
			if ($bid = ldap_bind($this->cid, $binddn, $bindpw)) {
				$this->error = "Success";
				$this->hascon=true;
			} else {
				$this->error = "Could not bind to " . $binddn;
			}
		} else {
			$this->error = "Could not connect to LDAP server";
		}
		error_reporting(E_ALL);
	}

	function disconnect()
	{
		if($this->hascon){
			ldap_close($this->cid);
			$this->hascon=false;
		}
	}

	function cd($dir)
	{
		if ($dir == "..")
			$this->basedn = $this->getParentDir();
		else
			$this->basedn = $dir;
	}

	function getParentDir($basedn = "")
	{
		if ($basedn=="")
			$basedn = $this->basedn;
		return(ereg_replace("[^,]*[,]*[ ]*(.*)", "\\1", $basedn));
	}

	function search($filter)
	{
		if($this->hascon){
			$this->clearResult();
			$e = error_reporting(0);
			$this->sr = ldap_search($this->cid, $this->basedn, $filter);
			$this->error = ldap_error($this->cid);
			$this->resetResult();
			error_reporting($e);
			$this->hasres=true;
			return($this->sr);
		}else{
			$this->error = "Could not connect to LDAP server";
			error_reporting($e);
			return("");
		}
	}

	function ls($filter = "(objectclass=*)", $basedn = "")
	{
		if($this->hascon){
			$this->clearResult();
			if ($basedn == "")
				$basedn = $this->basedn;
			$e = error_reporting(0);
			$this->sr = ldap_list($this->cid, $basedn, $filter);
			$ldap->error = ldap_error($this->cid);
			$this->resetResult();
			error_reporting($e);
			$this->hasres=true;
			return($this->sr);
		}else{
			$this->error = "Could not connect to LDAP server";
			error_reporting($e);
			return("");
		}
	}

	function cat($dn)
	{
		if($this->hascon){
			$e = error_reporting(0);
			$this->clearResult();
			$filter = "(objectclass=*)";
			$this->sr = ldap_read($this->cid, $dn, $filter);
			$ldap->error = ldap_error($this->cid);
			$this->resetResult();
			error_reporting($e);
			$this->hasres=true;
			return($this->sr);
		}else{
			$this->error = "Could not connect to LDAP server";
			error_reporting($e);
			return("");
		}
	}

	function fetch()
	{
		if($this->hascon){
			if($this->hasres){
				$e = error_reporting(0);
				if ($this->start == 0)
				{
					$this->start = 1;
					$this->re = ldap_first_entry($this->cid, $this->sr);
				} else {
					$this->re = ldap_next_entry($this->cid, $this->re);
				}
				if ($this->re)
				{
					$att = ldap_get_attributes($this->cid, $this->re);
				}
				$ldap->error = ldap_error($this->cid);
						if (!isset($att)){
						$att= array();
						}
				error_reporting($e);
				return($att);
			}else{
				$this->error = "Perform a Fetch with no Search";
				error_reporting($e);
				return("");
			}
		}else{
			$this->error = "Could not connect to LDAP server";
			error_reporting($e);
			return("");
		}
	}

	function resetResult()
	{
		$this->start = 0;
	}

	function clearResult()
	{
		if($this->hasres){
			$this->hasres = false;
			ldap_free_result($this->sr);
		}
	}

	function getDN()
	{
		if($this->hascon){
			if($this->hasres){
				$e = error_reporting(0);
				$rv = ldap_get_dn($this->cid, $this->re);
				$ldap->error = ldap_error($this->cid);
				error_reporting($e);
				return($rv);
			}else{
				$this->error = "Perform a Fetch with no Search";
				error_reporting($e);
				return("");
			}
		}else{
			$this->error = "Could not connect to LDAP server";
			error_reporting($e);
			return("");
		}
	}

	function count()
	{
		if($this->hascon){
			if($this->hasres){
				$e = error_reporting(0);
				$rv = ldap_count_entries($this->cid, $this->sr);
				$ldap->error = ldap_error($this->cid);
				error_reporting($e);
				return($rv);
			}else{
				$this->error = "Perform a Fetch with no Search";
				error_reporting($e);
				return("");
			}
		}else{
			$this->error = "Could not connect to LDAP server";
			error_reporting($e);
			return("");
		}
	}

	function rm($attrs = "", $dn = "")
	{
		if($this->hascon){
			if ($dn == "")
				$dn = $this->basedn;

			$e = error_reporting(0);
			$r = ldap_mod_del($this->cid, $dn, $attrs);
			$ldap->error = ldap_error($this->cid);
			error_reporting($e);
			return($r);
		}else{
			$this->error = "Could not connect to LDAP server";
			error_reporting($e);
			return("");
		}
	}

	function rename($attrs, $dn = "")
	{
		if($this->hascon){
			if ($dn == "")
				$dn = $this->basedn;

			$e = error_reporting(0);
			$r = ldap_mod_replace($this->cid, $dn, $attrs);
			$ldap->error = ldap_error($this->cid);
			error_reporting($e);
			return($r);
		}else{
			$this->error = "Could not connect to LDAP server";
			error_reporting($e);
			return("");
		}
	}

	function rmdir($deletedn)
	{
		if($this->hascon){
			$e = error_reporting(0);
			$r = ldap_delete($this->cid, $deletedn);
			$this->error = ldap_error($this->cid);
			error_reporting($e);
			return($r ? $r : 0);
		}else{
			$this->error = "Could not connect to LDAP server";
			error_reporting($e);
			return("");
		}
	}

	function modify($attrs)
	{
		if($this->hascon){
			$e = error_reporting(0);
			$r = ldap_modify($this->cid, $this->basedn, $attrs);
			$this->error = ldap_error($this->cid);
			error_reporting($e);
			return($r ? $r : 0);
		}else{
			$this->error = "Could not connect to LDAP server";
			error_reporting($e);
			return("");
		}
	}

	function add($attrs)
	{
		if($this->hascon){
			$e = error_reporting(0);
			$r = ldap_add($this->cid, $this->basedn, $attrs);
			$this->error = ldap_error($this->cid);
			error_reporting($e);
			return($r ? $r : 0);
		}else{
			$this->error = "Could not connect to LDAP server";
			error_reporting($e);
			return("");
		}
	}

}



?>
