<?php
require "apptabs.inc";

class applicationManagement extends plugin
{
  /* Dialog attributes */
  var $apptabs= NULL;
  var $applications= array();
  var $ui= NULL;
  var $acl= "";

  function applicationManagement ($config, $ui)
  {
	/* Save configuration for internal use */
	$this->config= $config;
	$this->ui= $ui;
  }

  function execute()
  {
        /* Check sorting variable */
	$this->check_sortranges();
	$this->reload();

	/* New application? */
	if (isset($_POST['new_app'])){

		/* By default we set 'dn' to 'new', all relevant plugins will
		   react on this. */
		$this->dn= "new";

		/* Create new usertab object */
		$this->apptabs= new apptabs($this->config,
				$this->config->data['TABS']['APPSTABS'], $this->dn);
		$this->apptabs->set_acl('#all#');
	}

	/* Cancel dialogs */
	if (isset($_POST['edit_cancel']) || isset($_POST['password_cancel'])){
		del_lock ($this->apptabs->dn);
		unset ($this->apptabs);
		$this->apptabs= NULL;
	}

	/* Finish apps edit is triggered by the tabulator dialog, so
	   the user wants to save edited data. Check and save at this
	   point. */
	if (isset($_POST['edit_finish'])){

		/* Check tabs, will feed message array */
		$this->apptabs->last= $this->apptabs->current;
		$this->apptabs->save_object();
		$message= $this->apptabs->check();

		/* Save, or display error message? */
		if (count($message) == 0){

			/* Save data data to ldap */
			$this->apptabs->save();

			/* Application has been saved successfully, remove lock from
			   LDAP. */
			if ($this->dn != "new"){
				del_lock ($this->dn);
			}

			/* There's no page reload so we have to read new apps at
			   this point. */
			$this->reload ();
			unset ($this->apptabs);
			$this->apptabs= NULL;

		} else {
			/* Ok. There seem to be errors regarding to the tab data,
			   show message and continue as usual. */
			show_errors($message);
		}
	}

	/* User wants to edit data? */
	if (isset($_POST['select_app']) && isset($_POST['applist']) && $_POST['applist'] != ""){

		/* Get 'dn' from posted 'applist', must be unique */
		$this->dn= $_POST['applist'];

		/* Check locking, save current plugin in 'back_plugin', so
		   the dialog knows where to return. */
		if (($user= get_lock($this->dn)) != ""){
			gen_locked_message ($user, $this->dn);
			return;
		}

		/* Lock the current entry, so everyone will get the
		   above dialog */
		add_lock ($this->dn, $this->ui->dn);

		/* Set up the users ACL's for this 'dn' */
		$acl= get_permissions ($this->dn, $this->ui->subtreeACL);
		$acl= get_module_permission($acl, "application", $this->dn);

		/* Register apptabs to trigger edit dialog */
		$this->apptabs= new apptabs($this->config,
				$this->config->data['TABS']['APPSTABS'], $this->dn);
		$this->apptabs->set_acl($acl);
	}

	/* Remove user was requested */
	if (isset($_POST['delete_app']) && isset($_POST['applist'])){

		/* Get 'dn' from posted 'uid' */
		$this->dn= $_POST['applist'];

		/* Load permissions for selected 'dn' and check if
		   we're allowed to remove this 'dn' */
		$acl= get_permissions ($this->dn, $this->ui->subtreeACL);
		$this->acl= get_module_permission($acl, "application", $this->dn);
		if (chkacl($this->acl, "delete") == ""){

			/* Check locking, save current plugin in 'back_plugin', so
			   the dialog knows where to return. */
			if (($user= get_lock($this->dn)) != ""){
				gen_locked_message ($user, $this->dn);
				exit ();
			}

			/* Lock the current entry, so nobody will edit it during deletion */
			add_lock ($this->dn, $this->ui->dn);
			require ("remove.inc");
			return;
		} else {

			/* Obviously the user isn't allowed to delete. Show message and
			   clean session. */
			print_red ("You are not allowed to delete this application!<br>");
		}
	}

	/* Confirmation for deletion has been passed. Group should be deleted. */
	if (isset($_POST['delete_app_confirm'])){

		/* Some nice guy may send this as POST, so we've to check
		   for the permissions again. */
		if (chkacl($this->acl, "delete") == ""){

			/* Delete request is permitted, perform LDAP action */
			$this->apptabs= new apptabs($this->config,
					$this->config->data['TABS']['APPSTABS'], $this->dn);
			$this->apptabs->set_acl($this->acl);
			$this->apptabs->delete ();
			unset ($this->apptabs);
			$this->apptabs= NULL;

			/* Group list has changed, reload it. */
			$this->reload ();
		} else {

			/* Normally this shouldn't be reached, send some extra
			   logs to notify the administrator */
			print_red ("You are not allowed to delete this application!<br>");
			gosa_log ("Warning: '".$this->ui->uid."' tried to trick group deletion.");
		}

		/* Remove lock file after successfull deletion */
		del_lock ($this->dn);
	}


	/* Delete application canceled? */
	if (isset($_POST['delete_cancel'])){
		del_lock ($this->dn);
	}

        /* Show tab dialog if object is present */
        if ($this->apptabs){
                $this->apptabs->execute();

                /* Don't show buttons if tab dialog requests this */
                if (!$this->apptabs->by_object[$this->apptabs->current]->dialog){
                        echo "<p style=\"text-align:right\">\n";
                        echo "<input type=submit name=\"edit_finish\" value=\""._("Finish")."\">\n";
                        echo "&nbsp;\n";
                        echo "<input type=submit name=\"edit_cancel\" value=\""._("Cancel")."\">\n";
                        echo "</p>";
                }
                return;
        }

	/* Show main page */
	require ("headpage.inc");
  }


  function reload()
  {
	/* Generate applications */
	$res= get_list($this->ui->subtreeACL, "(objectClass=gosaApplication)");
	$last_sorting= "invalid";

	$this->applications= array();
	foreach ($res as $value){
		if (preg_match ($this->sortranges[$this->sorting], $value["cn"][0])){
				$this->applications[$value["dn"]]= $value["cn"][0];
				if (isset($value["description"][0])){
					$this->applications[$value["dn"]]= $value["cn"][0]." (".
							utf8_decode($value["description"][0]).")";
				}
		}
	}
	natcasesort ($this->applications);
	reset ($this->applications);
  }

  function remove_from_parent()
  {
	/* Optionally execute a command after we're done */
	$this->postremove();
  }


  /* Save data to object */
  function save_object()
  {
  }


  /* Check values */
  function check()
  {
  }


  /* Save to LDAP */
  function save()
  {
	/* Optionally execute a command after we're done */
	$this->postcreate();
  }

  function adapt_from_template($dn)
  {
  }

  function password_change_needed()
  {
  }

  function show_header($button_text, $text, $disabled= FALSE)
  {
  }
}

?>
