<?php
require "grouptabs.inc";

class groupManagement extends plugin
{
  /* Dialog attributes */
  var $grouptab= NULL;
  var $grouplist= array();
  var $ui= NULL;
  var $acl= "";

  function groupManagement ($config, $ui)
  {
	/* Save configuration for internal use */
	$this->config= $config;
	$this->ui= $ui;
  }

  function execute()
  {
        /* Check sorting variable */
	$this->check_sortranges();
	$this->reload();

	/* New group? */
	if (isset($_POST['new_group'])){

		/* By default we set 'dn' to 'new', all relevant plugins will
		   react on this. */
		$this->dn= "new";

		/* Create new usertab object */
		$this->grouptab= new grouptabs($this->config,
				$this->config->data['TABS']['GROUPTABS'], $this->dn);
		$this->grouptab->set_acl('#all#');
	}

	/* Cancel dialogs */
	if (isset($_POST['edit_cancel']) || isset($_POST['password_cancel'])){
		del_lock ($this->grouptab->dn);
		unset ($this->grouptab);
		$this->grouptab= NULL;
	}

	/* Finish group edit is triggered by the tabulator dialog, so
	   the user wants to save edited data. Check and save at this
	   point. */
	if (isset($_POST['edit_finish'])){

		/* Check tabs, will feed message array */
		$message= $this->grouptab->check();

		/* Save, or display error message? */
		if (count($message) == 0){

			/* Save user data to ldap */
			$this->grouptab->save();

			/* Group has been saved successfully, remove lock from
			   LDAP. */
			if ($this->dn != "new"){
				del_lock ($this->dn);
			}

			/* There's no page reload so we have to read new users at
			   this point. */
			$this->reload ();
			unset ($this->grouptab);
			$this->grouptab= NULL;

		} else {
			/* Ok. There seem to be errors regarding to the tab data,
			   show message and continue as usual. */
			show_errors($message);
		}
	}

	/* User wants to edit data? */
	if (isset($_POST['select_group']) && isset($_POST['grouplist']) && $_POST['grouplist'] != ""){

		/* Get 'dn' from posted 'uid', must be unique */
		$this->dn= $_POST['grouplist'];

		/* Check locking, save current plugin in 'back_plugin', so
		   the dialog knows where to return. */
		if (($user= get_lock($this->dn)) != ""){
			gen_locked_message ($user, $this->dn);
			return;
		}

		/* Lock the current entry, so everyone will get the
		   above dialog */
		add_lock ($this->dn, $this->ui->dn);

		/* Set up the users ACL's for this 'dn' */
		$acl= get_permissions ($this->dn, $this->ui->subtreeACL);
		$acl= get_module_permission($acl, "group", $this->dn);

		/* Register grouptab to trigger edit dialog */
		$this->grouptab= new grouptabs($this->config,
				$this->config->data['TABS']['GROUPTABS'], $this->dn);
		$this->grouptab->set_acl($acl);
	}

	/* Remove user was requested */
	if (isset($_POST['delete_group']) && isset($_POST['grouplist'])){

		/* Get 'dn' from posted 'uid' */
		$this->dn= $_POST['grouplist'];

		/* Load permissions for selected 'dn' and check if
		   we're allowed to remove this 'dn' */
		$acl= get_permissions ($this->dn, $this->ui->subtreeACL);
		$this->acl= get_module_permission($acl, "group", $this->dn);
		if (chkacl($this->acl, "delete") == ""){

			/* Check locking, save current plugin in 'back_plugin', so
			   the dialog knows where to return. */
			if (($user= get_lock($this->dn)) != ""){
				gen_locked_message ($user, $this->dn);
				exit ();
			}

			/* Lock the current entry, so nobody will edit it during deletion */
			add_lock ($this->dn, $this->ui->dn);
			require ("remove.inc");
			return;
		} else {

			/* Obviously the user isn't allowed to delete. Show message and
			   clean session. */
			print_red ("You are not allowed to delete this group!<br>");
		}
	}

	/* Confirmation for deletion has been passed. Group should be deleted. */
	if (isset($_POST['delete_group_confirm'])){

		/* Some nice guy may send this as POST, so we've to check
		   for the permissions again. */
		if (chkacl($this->acl, "delete") == ""){

			/* Delete request is permitted, perform LDAP action */
			$this->grouptab= new grouptabs($this->config,
					$this->config->data['TABS']['GROUPTABS'], $this->dn);
			$this->grouptab->set_acl($this->acl);
			$this->grouptab->delete ();
			unset ($this->grouptab);
			$this->grouptab= NULL;

			/* Group list has changed, reload it. */
			$this->reload ();
		} else {

			/* Normally this shouldn't be reached, send some extra
			   logs to notify the administrator */
			print_red ("You are not allowed to delete this group!<br>");
			gosa_log ("Warning: '".$this->ui->uid."' tried to trick group deletion.");
		}

		/* Remove lock file after successfull deletion */
		del_lock ($this->dn);
	}


	/* Delete group canceled? */
	if (isset($_POST['delete_cancel'])){
		del_lock ($this->dn);
	}

        /* Show tab dialog if object is present */
        if ($this->grouptab){
                $this->grouptab->execute();

                /* Don't show buttons if tab dialog requests this */
                if (!$this->grouptab->by_object[$this->grouptab->current]->dialog){
                        echo "<p style=\"text-align:right\">\n";
                        echo "<input type=submit name=\"edit_finish\" value=\""._("Finish")."\">\n";
                        echo "&nbsp;\n";
                        echo "<input type=submit name=\"edit_cancel\" value=\""._("Cancel")."\">\n";
                        echo "</p>";
                }
                return;
        }

	/* Show main page */
	require ("headpage.inc");
  }


  function reload()
  {
	/* Generate grouplist */
	$res= get_list($this->ui->subtreeACL, "(objectClass=posixGroup)");
	$last_sorting= "invalid";

	/* Ignore primary user groups? */
	$ignoreGID= array();
	if (isset($_POST['noprimarygroups'])){
		$ldap= $this->config->get_ldap_link();
		$ldap->cd($this->config->current['BASE']);
		$ldap->search("(&(objectClass=posixAccount)(gidNumber=*))");
		while ($attrs= $ldap->fetch()){
			$ignoreGID[$attrs['gidNumber'][0]]= $attrs['uid'][0];
		}
	}

	$this->grouplist= array();
	foreach ($res as $value){
		if (isset($ignoreGID[$value['gidNumber'][0]])) continue;
		if (preg_match ($this->sortranges[$this->sorting], $value["cn"][0])){
				$this->grouplist[$value["dn"]]= $value["cn"][0];
				if (isset($value["description"][0])){
					$this->grouplist[$value["dn"]]= $value["cn"][0]." (".
							utf8_decode($value["description"][0]).")";
				}
		}
	}
	natcasesort ($this->grouplist);
	reset ($this->grouplist);
  }

  function remove_from_parent()
  {
	/* Optionally execute a command after we're done */
	$this->postremove();
  }


  /* Save data to object */
  function save_object()
  {
  }


  /* Check values */
  function check()
  {
  }


  /* Save to LDAP */
  function save()
  {
	/* Optionally execute a command after we're done */
	$this->postcreate();
  }

  function adapt_from_template($dn)
  {
  }

  function password_change_needed()
  {
  }

  function show_header($button_text, $text, $disabled= FALSE)
  {
  }
}

?>
