<?php
require "termtabs.inc";

class terminals extends plugin
{
  /* Dialog attributes */
  var $termtab= NULL;
  var $terminals= array();
  var $ui= NULL;
  var $acl= "";

  function terminals ($config, $ui)
  {
	/* Save configuration for internal use */
	$this->config= $config;
	$this->ui= $ui;
  }

  function execute()
  {
        /* Check sorting variable */
	$this->check_sortranges();
	$this->reload();

	/* Create new default terminal */
	if (isset($_POST['new_terminal'])) {
		$this->dn= "new";
		$this->acl= "#all#";
	
		$this->termtab= new termtabs($this->config,
			$this->config->data['TABS']['TERMTABS'], $this->dn);
		$this->termtab->set_acl ($this->acl);
		$this->termtab->by_object['termgeneric']->cn= "default";
		$this->termtab->by_object['termservice']->cn= "default";
	}

	/* User wants to edit data? */
	if (isset($_POST['select_terminal']) && isset($_POST['termlist']) &&
		 $_POST['termlist'] != ""){

		/* Get 'dn' from posted 'cn', must be unique */
		$this->dn= $_POST['termlist'];

		/* Check locking, save current plugin in 'back_plugin', so
		   the dialog knows where to return. */
		if (($user= get_lock($this->dn)) != ""){
			gen_locked_message ($user, $this->dn);
			return;
		}

		/* Lock the current entry, so everyone will get the
		   above dialog */
		add_lock ($this->dn, $this->ui->dn);

		/* Set up the users ACL's for this 'dn' */
		$acl= get_permissions ($this->dn, $this->ui->subtreeACL);
		$acl= get_module_permission($acl, "terminal", $this->dn);

		/* Register termtab to trigger edit dialog */
		$this->termtab= new termtabs($this->config,
					$this->config->data['TABS']['TERMTABS'], $this->dn);
		$this->termtab->set_acl($acl);
	}

	/* Set terminals root password */
	if (isset($_POST['set_password']) && isset($_POST['termlist'])){
		$this->dn= $_POST['termlist'];

		/* Check if user is allowed to set password */
		$acl= get_permissions ($this->dn, $this->ui->subtreeACL);
		$acl= get_module_permission($acl, "terminal", $this->dn);
		if (chkacl($acl, "password") == ""){
			require ("password.inc");
			return;
		} else {
			print_red ("You are not allowed to set this systems password!<br>");
		}
	}


	/* Delete terminal canceled? */
	if (isset($_POST['delete_cancel']) || isset($_POST['password_cancel'])){
		del_lock ($this->dn);
	}

	/* Remove terminal was requested */
	if (isset($_POST['delete_terminal']) && isset($_POST['termlist'])){

		/* Get 'dn' from posted termlinst */
		$this->dn= $_POST['termlist'];

		/* Load permissions for selected 'dn' and check if
		   we're allowed to remove this 'dn' */
		$acl= get_permissions ($this->dn, $this->ui->subtreeACL);
		$this->acl= get_module_permission($acl, "terminal", $this->dn);
		if (chkacl($this->acl, "delete") == ""){

			/* Check locking, save current plugin in 'back_plugin', so
			   the dialog knows where to return. */
			if (($user= get_lock($this->dn)) != ""){
				gen_locked_message ($user, $this->dn);
				exit ();
			}

			/* Lock the current entry, so nobody will edit it during deletion */
			add_lock ($this->dn, $this->ui->dn);
			require ("remove.inc");
			return;
		} else {

			/* Obviously the user isn't allowed to delete. Show message and
			   clean session. */
			print_red ("You are not allowed to delete this user!<br>");
		}
	}

	/* Confirmation for deletion has been passed. Terminal should be deleted. */
	if (isset($_POST['delete_terminal_confirm'])){

		/* Some nice guy may send this as POST, so we've to check
		   for the permissions again. */
		if (chkacl($this->acl, "delete") == ""){

			/* Delete request is permitted, perform LDAP action */
			$this->termtab= new termtabs($this->config,
				$this->config->data['TABS']['TERMTABS'], $this->dn);
			$this->termtab->set_acl($this->acl);
			$this->termtab->by_object['termgeneric']->remove_from_parent ();
			unset ($this->termtab);
			$this->termtab= NULL;

			/* Terminal list has changed, reload it. */
			$this->reload ();
		} else {

			/* Normally this shouldn't be reached, send some extra
			   logs to notify the administrator */
			print_red ("You are not allowed to delete this terminal!<br>");
			gosa_log ("Warning: '".$this->ui->uid."' tried to trick terminal ".
				"deletion.");
		}

		/* Remove lock file after successfull deletion */
		del_lock ($this->dn);
	}

	/* Finish user edit is triggered by the tabulator dialog, so
	   the user wants to save edited data. Check and save at this
	   point. */
	if (isset($_POST['edit_finish'])){

		/* Check tabs, will feed message array */
		$message= $this->termtab->check(TRUE);

		/* Save, or display error message? */
		if (count($message) == 0){

			/* Save terminal data to ldap */
			$this->termtab->save();

			/* Terminal has been saved successfully, remove lock from
			   LDAP. */
			if ($this->dn != "new"){
				del_lock ($this->dn);
			}

			/* There's no page reload so we have to read new terminals at
			   this point. */
			$this->reload ();
			unset ($this->termtab);
			$this->termtab= NULL;

		} else {
			/* Ok. There seem to be errors regarding to the tab data,
			   show message and continue as usual. */
			show_errors($message);
		}

	}

	/* Cancel dialogs */
	if (isset($_POST['edit_cancel']) || isset($_POST['password_cancel'])){
		del_lock ($this->termtab->dn);
		unset ($this->termtab);
		$this->termtab= NULL;
	}

	/* Show tab dialog if object is present */
	if ($this->termtab){
		$this->termtab->execute();

		/* Don't show buttons if tab dialog requests this */
		if (!$this->termtab->by_object[$this->termtab->current]->dialog){
			echo "<p style=\"text-align:right\">\n";
			echo "<input type=submit name=\"edit_finish\" value=\""._("Finish")."\">\n";
			echo "&nbsp;\n";
			echo "<input type=submit name=\"edit_cancel\" value=\""._("Cancel")."\">\n";
			echo "</p>";
		}
		return;
	}

	/* Show main page */
	require ("headpage.inc");
  }


  function remove_from_parent()
  {
	/* Optionally execute a command after we're done */
	$this->postremove();
  }


  /* Save data to object */
  function save_object()
  {
  }


  /* Check values */
  function check()
  {
  }


  /* Save to LDAP */
  function save()
  {
  }

  function adapt_from_template($dn)
  {
  }

  function password_change_needed()
  {
  }

  function show_header($button_text, $text, $disabled= FALSE)
  {
  }

  function reload()
  {
	/* Load terminal shortcuts */
	$responsible= array();
	foreach ($this->config->departments as $key => $value){
		if (get_module_permission(get_permissions ($value, $this->ui->subtreeACL),
			      "terminal", $value) == "#all#"){
			$responsible[$key]= $value;
		}
	}
	$this->last_sorting= "invalid";

	/* Get all gotoTerminal's */
	$this->terminals= array();
	$res= get_list($this->ui->subtreeACL, "(objectClass=gotoTerminal)");
	foreach ($res as $value){

		/* Look for new terminals and mark them with '+' */
		$tmp= preg_replace("/^[^,]+,[^o]*ou=terminals,ou=systems,/", "", $value["dn"]);
		if (preg_match ("/,ou=incoming,/", $tmp)){
			$add= "+"._("New terminal");
		} else {
			$add= "";
		}

		if (in_array ($tmp, $responsible) || $add != ""){
			preg_match ($this->sortranges[$this->sorting], $value["cn"][0]);

			if (isset($value["macAddress"][0]) &&
				$value["macAddress"][0] != "00:00:00:00:00"){

				$this->terminals[$value["dn"]]= $add." ".
						$value["cn"][0].
						" [".$value["macAddress"][0]."]";
			} else {
				$location= array_search($tmp, $this->config->departments);
				$this->terminals[$value["dn"]]= "-"._("Default for").
								" $location";
			}
		}
	}
	natcasesort ($this->terminals);
	reset ($this->terminals);
  }


}

?>
