<?php
class termservice extends plugin
{
  /* Generic terminal attributes */
  var $gotoLpdEnable= FALSE;
  var $gotoXMonitor= "";
  var $gotoXMethod= "query";
  var $gotoXdmcpServer= "";
  var $gotoFontPath= "";
  var $gotoXDriver= "";
  var $gotoXResolution= "";
  var $gotoXColordepth= "";
  var $gotoXHsync= "";
  var $gotoXVsync= "";
  var $gotoLpdEnable= "";
  var $gotoLpdServer= "";
  var $gotoScannerEnable= "";
  var $gotoScannerModel= "";
  var $gotoScannerClients= "";
  var $gotoScannerBackend= "";

  /* Needed values and lists */
  var $base= "";
  var $cn= "";
  var $XMethods= array ("indirect" => "show chooser", "query" => "direct" ,"load" => "load balanced",
                        "secure" => "load balanced via ssh");
  var $XDrivers= array("unknown", "ati", "atimisc", "chips", "cirrus", "cyrix", "fbdev",
                       "i128", "i740", "i810", "imstt", "mga", "neomagic", "newport", "nv",
                       "r128", "radeon", "rendition", "s3", "s3virge", "savage", "siliconmotion",
                       "sis", "tdfx", "tga", "trident", "tseng", "vesa", "vga");
  var $XResolutions= array("default", "640x480", "800x600", "1024x768", "1152x864", "1280x1024",
                           "1400x1050", "1600x1200");
  var $XColordepths= array("default", "8", "15", "16", "24");
  var $gotoScannerModels= array();


  /* attribute list for save action */
  var $attributes= array("cn", "gotoLpdEnable", "gotoXMonitor", "gotoXMethod", "gotoXdmcpServer",
			"gotoFontPath", "gotoXDriver", "gotoXResolution", "gotoXColordepth",
			"gotoXHsync", "gotoXVsync", "gotoLpdEnable", "gotoLpdServer",
			"gotoScannerEnable", "gotoScannerModel", "gotoScannerClients",
			"gotoScannerBackend");
  var $objectclasses= array("gotoTerminal", "goHard");


  function termservice ($config, $dn= NULL)
  {
	plugin::plugin ($config, $dn);

	/* Load scanner models */
	$fcontents = file ($this->config->basedir."/include/scanner.inc");
	while (list ($line_num, $line) = each ($fcontents)) {
		preg_match('/^(\w+) "([^"]+)" "([^"]+)" "([^"]+)"/', "$line", $matches);
		$this->gotoScannerModels[$matches[2]." - ".$matches[3]]= $matches[1]."|".$matches[4];
	}
	ksort ($this->gotoScannerModels);
  }

  function execute()
  {
	/* Do we need to flip is_account state? */
	if (isset($_POST['modify_state'])){
		$this->is_account= !$this->is_account;
	}

	/* Do we represent a valid terminal? */
	if (!$this->is_account && $this->parent == NULL){
		echo "<img src=\"images/stop.png\" align=center>&nbsp;<b>".
		_("This 'dn' has no terminal features.")."</b>";
		return;
	}

	/* Show main page */
	require ("service.inc");
  }

  function remove_from_parent()
  {
	$ldap= $this->config->get_ldap_link();
	$ldap->rmdir($this->dn);
	show_ldap_error($ldap->error);
  }


  /* Save data to object */
  function save_object()
  {
	plugin::save_object();

	/* Save checkbox state */
	if (isset ($_POST['gotoXMethod'])){
		foreach (array("gotoLpdEnable", "gotoScannerEnable") as $val){
			
			if (!isset ($_POST["$val"]) && chkacl ($this->acl, "$val") == ""){
				$this->$val= FALSE;
			} else {
				$this->$val= TRUE;
			}
		}
	}
  }


  /* Check supplied data */
  function check()
  {
	$message= array();

	/* Default entries can use blank hsync/vsync entries */
	if ($this->dn != "" && $this->cn != "default"){

		/* Check vsync for correct usage */
		$val= preg_replace ("/\s/", "", $this->gotoXVsync);
		if (!preg_match ("/^\d+(\.\d+)?(\-\d+(\.\d+)?)?$/", $val)
			&& chkacl ($this->acl, "gotoXVsync") == ""){

			$message[]= _("Please specify a valid VSync range.");
		} elseif (chkacl ($this->acl, "gotoXVsync") == ""){
			list($v1,$v2)= split ("-", $val);
			if ($v2 != ""){
				if ($v1 > $v2){
					$message[]= _("Please specify a valid VSync range.");
				}
			}
		}

		/* Check hsync for correct usage */
		$val= preg_replace ("/\s/", "", $this->gotoXHsync);
		if (!preg_match ("/^\d+(\.\d+)?(\-\d+(\.\d+)?)?$/", $val)
			&& chkacl ($this->acl, "gotoXHsync") == ""){

			$message[]= _("Please specify a valid HSync range.");
		} elseif (chkacl ($this->acl, "gotoXHsync") == ""){
			list($v1,$v2)= split ("-", $val);
			if ($v2 != ""){
				if ($v1 > $v2){
					$message[]= _("Please specify a valid HSync range.");
				}
			}
		}
	}
	return ($message);
  }


  /* Save to LDAP */
  function save()
  {
	plugin::save();

	/* Strip out 'default' values */
	foreach (array("gotoXdmcpServer", "gotoFontPath", "gotoXResolution",
			"gotoXColordepth", "gotoLpdServer") as $val){

		if ($this->attrs[$val] == "default"){
			$this->attrs[$val]= array();
		}
	}

	/* Write back to ldap */
	$ldap= $this->config->get_ldap_link();
	$ldap->cd($this->dn);
	$ldap->modify($this->attrs);
	show_ldap_error($ldap->error);
  }

}

?>
