<?php
require "usertabs.inc";

class userManagement extends plugin
{
  /* Dialog attributes */
  var $usertab= NULL;
  var $userlist= array();
  var $ui= NULL;
  var $acl= "";
  var $templates= array();
  var $lognames= array();

  function userManagement ($config, $ui)
  {
	/* Save configuration for internal use */
	$this->config= $config;
	$this->ui= $ui;
  }

  function execute()
  {
        /* Check sorting variable */
	$this->check_sortranges();
	$this->reload();

	/* React on user interaction here */
	if ($_SERVER["REQUEST_METHOD"] == "POST"){

		/* User wants to edit data? */
		if (isset($_POST['select_user']) && isset($_POST['userlist']) && $_POST['userlist'] != ""){

			/* Get 'dn' from posted 'uid', must be unique */
			$this->dn= get_dn ($_POST['userlist']);

			/* Check locking, save current plugin in 'back_plugin', so
			   the dialog knows where to return. */
			if (($user= get_lock($this->dn)) != ""){
				gen_locked_message ($user, $this->dn);
				return;
			}

			/* Lock the current entry, so everyone will get the
			   above dialog */
			add_lock ($this->dn, $this->ui->dn);

			/* Set up the users ACL's for this 'dn' */
			$acl= get_permissions ($this->dn, $this->ui->subtreeACL);
			$acl= get_module_permission($acl, "user", $this->dn);

			/* Register usertab to trigger edit dialog */
			$this->usertab= new usertabs($this->config, 
				$this->config->data['TABS']['USERTABS'], $this->dn);
			$this->usertab->set_acl($acl);
		}
	}

	/* Reset requested? */
	if (isset($_POST['edit_cancel']) || isset($_POST['password_cancel'])){
		del_lock ($this->usertab->dn);
		unset ($this->usertab);
		$this->usertab= NULL;
		$this->lognames= array();;
		$this->sn= "";
		$this->givenName= "";
		$this->uid= "";
	}

	/* Password change requested */
	if (isset($_POST['setpass_user']) && isset($_POST['userlist'])){

		/* Get 'dn' from posted 'uid' */
		$this->dn= get_dn ($_POST['userlist']);

		/* Load permissions for selected 'dn' and check if
		   we're allowed to remove this 'dn' */
		$acl= get_permissions ($this->dn, $this->ui->subtreeACL);
		$acl= get_module_permission($acl, "user", $this->dn);
		if (chkacl($acl, "password") == ""){

			/* User is allowed to change passwords, save 'dn' and 'acl' for next
			   dialog. */
			$this->acl= $acl;
			require ("password.inc");
			return;

		} else {
			/* User is not allowed. Show message and cancel. */
			print_red ("You are not allowed to set this users password!<br>");
		}
	}

	/* Perform password change */
	if (isset($_POST['password_finish'])){

		/* For security reasons, check if user is allowed to set password again */
		if (chkacl($this->acl, "password") == "" || chkacl($this->acl, "create")){

			/* Check input and feed errors into 'message' */
			$message= array();

			/* Sanity checks... */
			if ($_POST['new_password'] != $_POST['repeated_password']){

				/* Matching passwords in new and repeated? */
				$message[]= _("The passwords you've entered as ".
						"'New password' and 'Repeated new password' do not match.");
			} else {

				/* Empty password is not permitted by default. */
				if ($_POST['new_password'] == ""){
					$message[]= _("The password you've entered as 'New password' is empty.");
				}
			}

			/* Errors, or change password? */
			if (count($message) != 0){

				/* Show error message and continue editing */
				show_errors ($message);
				require ("password.inc");
				return;
			}

			/* Set password, perform required steps */
			if ($this->usertab && $this->usertab->password_change_needed()){
				$obj= $this->usertab->by_object['user'];
				change_password ($this->usertab->dn, $_POST['new_password'],
						0, $obj->pw_storage);
				unset($this->usertab);
				$this->usertab= NULL;
			} else {
				change_password ($this->dn, $_POST['new_password']);
			}
		} else {

			/* Missing permissions, show message */
			print_red ("You are not allowed to set this users password!<br>");
		}

		/* Clean session, delete lock */
		del_lock ($this->dn);
		unset ($this->usertab);
                $this->usertab= NULL;
                $this->lognames= array();;
                $this->sn= "";
                $this->givenName= "";
                $this->uid= "";
	}


	/* Remove user was requested */
	if (isset($_POST['delete_user']) && isset($_POST['userlist'])){

		/* Get 'dn' from posted 'uid' */
		$this->dn= get_dn($_POST['userlist']);

		/* Load permissions for selected 'dn' and check if
		   we're allowed to remove this 'dn' */
		$acl= get_permissions ($this->dn, $this->ui->subtreeACL);
		$this->acl= get_module_permission($acl, "user", $this->dn);
		if (chkacl($this->acl, "delete") == ""){

			/* Check locking, save current plugin in 'back_plugin', so
			   the dialog knows where to return. */
			if (($user= get_lock($this->dn)) != ""){
				gen_locked_message ($user, $this->dn);
				exit ();
			}

			/* Lock the current entry, so nobody will edit it during deletion */
			add_lock ($this->dn, $this->ui->dn);
			require ("remove.inc");
			return;
		} else {

			/* Obviously the user isn't allowed to delete. Show message and
			   clean session. */
			print_red ("You are not allowed to delete this user!<br>");
		}
	}
	


	/* Confirmation for deletion has been passed. User should be deleted. */
	if (isset($_POST['delete_user_confirm'])){

		/* Some nice guy may send this as POST, so we've to check
		   for the permissions again. */
		if (chkacl($this->acl, "delete") == ""){

			/* Delete request is permitted, perform LDAP action */
			$this->usertab= new usertabs($this->config, $this->config->data['TABS']['USERTABS'],
						$this->dn);
			$this->usertab->set_acl($this->acl);
			$this->usertab->delete ();
			unset ($this->usertab);
			$this->usertab= NULL;

			/* User list has changed, reload it. */
			$this->reload ();
		} else {

			/* Normally this shouldn't be reached, send some extra
			   logs to notify the administrator */
			print_red ("You are not allowed to delete this user!<br>");
			gosa_log ("Warning: '".$this->ui->uid."' tried to trick user deletion.");
		}

		/* Remove lock file after successfull deletion */
		del_lock ($this->dn);
	}


	/* Delete user canceled? */
	if (isset($_POST['delete_cancel'])){
		del_lock ($this->dn);
	}


	/* Finish user edit is triggered by the tabulator dialog, so
	   the user wants to save edited data. Check and save at this
	   point. */
	if (isset($_POST['edit_finish'])){

		/* Check tabs, will feed message array */
		$this->usertab->last= $this->usertab->current;
		$this->usertab->save_object();
		$message= $this->usertab->check();

		/* Save, or display error message? */
		if (count($message) == 0){

			/* No errors. Go ahead and prepare to ask for a password
			  in case we're creating a new user. 'dn' will be 'new'
			  in this case. It is set to the correct value later. */
			if ($this->dn == "new"){
				$set_pass= 1;
			} else {
				$set_pass= 0;
			}

			/* Save user data to ldap */
			$this->usertab->save();

			/* User has been saved successfully, remove lock from
			   LDAP. */
			if ($this->dn != "new"){
				del_lock ($this->dn);
			}

			/* In case of new users, ask for a password */
			if ($set_pass || $this->usertab->password_change_needed()){

				require ("password.inc");
				return;
			}

			/* There's no page reload so we have to read new users at
			   this point. */
			$this->reload ();
			unset ($this->usertab);
			$this->usertab= NULL;

		} else {
			/* Ok. There seem to be errors regarding to the tab data,
			   show message and continue as usual. */
			show_errors($message);
		}
	}


	/* New user/template request */
	if (isset($_POST['new_user']) || isset($_POST['new_template'])){

		/* By default we set 'dn' to 'new', all relevant plugins will
		   react on this. */
		$this->dn= "new";

		/* Create new usertab object */
		$this->usertab= new usertabs($this->config,
			 $this->config->data['TABS']['USERTABS'], $this->dn);
		$this->usertab->set_acl('#all#');

		/* Take care about templates */
		if (isset($_POST['new_template'])){
			$this->is_template= TRUE;
			$this->usertab->set_template_mode ();
		}

		/* Use template if there are any of them */
		if (count($this->templates) && !isset($_POST['new_template'])){
			require_once ("choose_template.inc");
			return;
		}

	}

	/* Continue template editing */
	if (isset($_POST['template_continue']) && $_POST['template'] != 'none' && !isset($_POST['uid'])){
		$this->sn= $_POST['name'];
		$this->givenName= $_POST['givenName'];

		/* Check for requred values */
		$message= array();
		if ($this->sn == "") {
			$message[]= _("The required field 'Name' is not set.");
		}
		if ($this->givenName == "") {
			$message[]= _("The required field 'Given name' is not set.");
		}

		/* Check if dn is used */
		$dn= preg_replace("/^[^,]+,/i", "", $_POST['template']);
		$ldap= $this->config->get_ldap_link();
		$ldap->cd ($dn);
		$ldap->search ("(&(sn=".$this->sn.")(givenName=".$this->givenName."))");
		if ($ldap->count () != 0){
			$message[]= _("A person with the choosen name is already used in this tree.");
		}
		
		/* Show error message / continue editing */
		if (count($message) > 0){
			show_errors ($message);
		} else {
			$this->lognames= gen_uid ($this->sn, $this->givenName);
		}

		require_once ("choose_template.inc");
		return;
	}

	/* Finish template preamble */
	if (isset($_POST['template_continue']) && $_POST['template'] != 'none' && isset($_POST['uid'])){

		/* Move user supplied data to sub plugins */
		$this->uid= $_POST['uid'];
		$this->usertab->uid= $this->uid;
		$this->usertab->sn= $this->sn;
		$this->usertab->givenName= $this->givenName;
		$this->usertab->givenName= $this->givenName;
		$this->usertab->adapt_from_template($_POST['template']);
	}

	/* Show tab dialog if object is present */
	if ($this->usertab){
		$this->usertab->execute();

		/* Don't show buttons if tab dialog requests this */
		if (!$this->usertab->by_object[$this->usertab->current]->dialog){
			echo "<p style=\"text-align:right\">\n";
			echo "<input type=submit name=\"edit_finish\" value=\""._("Finish")."\">\n";
			echo "&nbsp;\n";
			echo "<input type=submit name=\"edit_cancel\" value=\""._("Cancel")."\">\n";
			echo "</p>";
		}
		return;
	}

	/* Show main page */
	require ("headpage.inc");
  }


  function reload()
  {
	/* Generate userlist */
	$this->list= get_list($this->ui->subtreeACL, "(objectClass=gosaAccount)");
	$this->userlist= array();

	foreach ($this->list as $value){
		if (preg_match ($this->sortranges[$this->sorting], $value["sn"][0])){
			if (isset($value["uid"][0])){
				if (isset($value["givenName"][0])){
					$this->userlist[$value["uid"][0]]= utf8_decode($value["sn"][0]).", ".
									utf8_decode($value["givenName"][0]).
									" [".$value["uid"][0]."]";
				} else {
					$this->userlist[$value["uid"][0]]= utf8_decode($value["sn"][0]).
									" [".$value["uid"][0]."]";
				}
			}
		}
	}

	/* Generate userlist */
	$this->list= get_list($this->ui->subtreeACL, "(objectClass=gosaUserTemplate)");
	foreach ($this->list as $value){
		if (preg_match ($this->sortranges[$this->sorting], $value["cn"][0])){
			$this->userlist[$value["uid"][0]]= "* ".$value["uid"][0]." ("._("Template").")";
		}
	}

	/* Sort list and save */
	natcasesort ($this->userlist);
	reset ($this->userlist);

	/* Generate template list */
	$this->templates= array();

	$ldap= $this->config->get_ldap_link();
	foreach ($this->config->departments as $key => $value){
		$acl= get_permissions ("ou=people,$value", $this->ui->subtreeACL);
		$acl= get_module_permission($acl, "user", "ou=people,$value");
		if (chkacl($acl, "create") == ""){
			$ldap->cd ("ou=people,$value");
			$ldap->search ("(objectClass=gosaUserTemplate)");

			if ($ldap->count() != 0){
				while ($attrs= $ldap->fetch()){
					$this->templates[$ldap->getDN()]=
						$attrs['uid'][0]." - $key";
				}
				$this->templates['none']= _("none");
			}
		}
	}
	natcasesort ($this->templates);
        reset ($this->templates);
  }

  function remove_from_parent()
  {
	/* Optionally execute a command after we're done */
	$this->postremove();
  }


  /* Check values */
  function check()
  {
  }


  /* Save to LDAP */
  function save()
  {
	/* Optionally execute a command after we're done */
	$this->postcreate();
  }

  function adapt_from_template($dn)
  {
  }

  function password_change_needed()
  {
  }

  function show_header($button_text, $text, $disabled= FALSE)
  {
  }
}

?>
