<?php
class gofaxAccount extends plugin
{
  /* Fax attributes */
  var $goFaxDeliveryMode= "";
  var $facsimileTelephoneNumber= "";
  var $goFaxIsEnabled= 1;
  var $goFaxPrinter= "";
  var $goFaxDivertNumber= "";
  var $goFaxLanguage= "";
  var $goFaxFormat= "";
  var $goFaxRBlocklist = array();
  var $goFaxRBlockgroups= array();
  var $goFaxSBlocklist= array();
  var $goFaxSBlockgroups= array();
  var $facsimileAlternateTelephoneNumber= array();

  /* Internal variables */
  var $printerList= array();
  var $has_mailAccount= FALSE;
  var $locals_dialog= FALSE;
  var $in_blocklist_dialog= FALSE;
  var $out_blocklist_dialog= FALSE;
  var $current_blocklist= array();

  /* attribute list for save action */
  var $attributes= array("goFaxDeliveryMode", "goFaxIsEnabled",
			"goFaxPrinter", "goFaxDivertNumber", "goFaxLanguage", "goFaxFormat");
  var $objectclasses= array("goFaxAccount");

  function gofaxAccount ($config, $dn= NULL)
  {
	/* General initialization */
	plugin::plugin ($config, $dn);

	if ($dn != "new"){
		/* Get arrays */
		foreach (array("goFaxRBlocklist", "goFaxRBlockgroups", "goFaxSBlocklist",
			"goFaxSBlockgroups", "facsimileAlternateTelephoneNumber") as $val){

			if (isset($this->attrs["$val"]["count"])){
				for ($i= 0; $i<$this->attrs["$val"]["count"]; $i++){
					array_push($this->$val, utf8_decode($this->attrs["$val"][$i]));
				}
			}
		}

		/* Set up has_mailAccount */
		if (in_array("gosaMailAccount", $this->attrs['objectClass'])){
			$this->has_mailAccount= TRUE;
		}
	}

	/* Load printer list */
	if (isset($this->config->data['SERVERS']['CUPS'])){
		$this->printerList= get_printer_list ($this->config->data['SERVERS']['CUPS']);
		asort ($this->printerList);
	}
  }

  function execute()
  {
	/* Check sorting variable */
	$this->check_sortranges();

	$ldap= $this->config->get_ldap_link();

	/* Do we need to flip is_account state? */
	if (isset($_POST['modify_state'])){
		$this->is_account= !$this->is_account;
	}

	/* Do we represent a valid account? */
	if (!$this->is_account && $this->parent == NULL){
		echo "<img src=\"images/stop.png\" align=center>&nbsp;<b>".
		_("This 'dn' has no fax extensions.")."</b>";
		return;
	}

	/* Show tab dialog headers */
	if ($this->parent != NULL){
		if ($this->is_account){
			$this->show_header(_("Remove fax account"),
				_("This account has fax features enabled. ".
				  "You can disable them by clicking below."));
		} else {
			$this->show_header(_("Create fax account"),
				_("This account has fax features disabled. ".
				  "You can enable them by clicking below."));
			return;
		}
	}

	/* Trigger Add local fax alternatives dialog */
	if (isset($_POST['add_local_alternate'])){
		$this->locals_dialog= TRUE;	
		$this->dialog= TRUE;	
	}

	/* Add alternatives from dialog */
	if (isset($_POST['add_locals_finish'])){
		if (isset($_POST['local_list']) &&
			chkacl ($this->acl, "facsimileAlternateTelephoneNumber") == ""){

			foreach ($_POST['local_list'] as $val){
				$this->addAlternate($val);
			}
		}
	}

	/* Add alternatives */
	if (isset($_POST['add_alternate'])){
		if ($_POST['forward_address'] != "" &&
			is_phone_nr($_POST['forward_address']) &&
			chkacl ($this->acl, "facsimileAlternateTelephoneNumber") == ""){

			$this->addAlternate($_POST['forward_address']);
		}
	}

	/* Delete alternate fax number */
	if (isset($_POST['delete_alternate'])){
		if (isset($_POST['alternate_list']) && count($_POST['alternate_list']) &&
			chkacl ($this->acl, "facsimileAlternateTelephoneNumber") == ""){

			$this->delAlternate ($_POST['alternate_list']);
		}
	}


	/* Edit incoming blocklists */
	if (isset($_POST['edit_incoming'])){
		$this->current_blocklist= array_merge($this->goFaxRBlocklist,
						$this->goFaxRBlockgroups);
		sort($this->current_blocklist);
		reset($this->current_blocklist);

		$this->in_blocklist_dialog= TRUE;
		$this->dialog= TRUE;
	}

	/* Edit outgoing blocklists */
	if (isset($_POST['edit_outgoing'])){
		$this->current_blocklist= array_merge($this->goFaxSBlocklist,
						$this->goFaxSBlockgroups);
		sort($this->current_blocklist);
		reset($this->current_blocklist);

		$this->out_blocklist_dialog= TRUE;
		$this->dialog= TRUE;
	}

	/* Add number to blocklist (dialog) */
	if (isset($_POST['add_blocklist_number']) && $_POST['block_number'] != ""){
		if (!is_phone_nr($_POST['block_number'])){
			print_red ("You're trying to add an invalid phone number.");
		} else {
			array_push($this->current_blocklist, $_POST['block_number']);
			$this->current_blocklist= array_unique($this->current_blocklist);
			sort($this->current_blocklist);
			reset($this->current_blocklist);
		}
	}

	/* Add list to blocklist */
	if (isset($_POST['add_blocklist']) && isset($_POST['predefined_list'])){
		$this->current_blocklist= array_merge($this->current_blocklist,
						$_POST['predefined_list']);
		$this->current_blocklist= array_unique($this->current_blocklist);
		sort($this->current_blocklist);
		reset($this->current_blocklist);
	}

	/* Delete from blocklist */
	if (isset($_POST['delete_blocklist_number']) && isset($_POST['block_list'])){
		$tmp= array();
		foreach($this->current_blocklist as $val){
			if (!in_array($val, $_POST['block_list'])){
				$tmp[]= $val;
			}
		}
		$this->current_blocklist= $tmp;
	}

	/* Blocklist edit finished */
	if (isset($_POST['edit_blocklists_finish'])){

		/* Incoming or outgoing? */
		if ($this->in_blocklist_dialog){
			$destlist="goFaxRBlocklist";
			$destgroup="goFaxRBlockgroups";
		} else {
			$destlist="goFaxSBlocklist";
			$destgroup="goFaxSBlockgroups";
		}

		/* Transfer values to ourself */
		$this->$destlist= array();
		$this->$destgroup= array();

		foreach ($this->current_blocklist as $val){
			if (is_phone_nr($val)){
				array_push($this->$destlist, $val);
			} else {
				array_push($this->$destgroups, $val);
			}
		}
	}

	/* Set departments */
	if ($this->locals_dialog ||
		$this->in_blocklist_dialog ||
		$this->out_blocklist_dialog){

		$list= array ();
		if (isset ($_POST['department'])){
			$ldap->cd ($this->config->departments[$_POST['department']]);
		} else {
			$ldap->cd ($this->config->current['BASE']);
		}
	}

	/* Cancel  dialogs */
	if (isset($_POST['add_locals_cancel']) || isset($_POST['edit_blocklists_finish']) ||
		isset($_POST['edit_blocklists_cancel']) || isset($_POST['add_locals_finish'])){

		$this->locals_dialog= FALSE;
		$this->in_blocklist_dialog= FALSE;
		$this->out_blocklist_dialog= FALSE;
		$this->dialog= FALSE;
	}

	/* Manage locals dialog */
	if ($this->locals_dialog){
		$ldap->search ("(objectClass=goFaxAccount)");
		while ($attrs= $ldap->fetch()){
			if (preg_match ($this->sortranges[$this->sorting], $attrs["sn"][0])){
				$list[$attrs['facsimileTelephoneNumber'][0]]=
						utf8_decode($attrs['sn'][0]).", ".
						utf8_decode($attrs['givenName'][0])." [".
						$attrs['facsimileTelephoneNumber'][0]."]";
			}
		}

		/* Show dialog */
		require_once ("edit_locals.inc");
		return;
	}

	/* Manage incoming blocklists */
	if ($this->in_blocklist_dialog){
		$ldap->search ("(objectClass=goFaxRBlock)");
		while ($attrs= $ldap->fetch()){
			$list[$attrs['cn'][0]]=
				utf8_decode($attrs['description'][0]).
				" [".$attrs['cn'][0]."]";
		 }

		/* Show dialog */
		require_once ("edit_lists.inc");
		return;
	}

	/* Manage outgoing blocklists */
	if ($this->out_blocklist_dialog){
		$ldap->search ("(objectClass=goFaxSBlock)");
		while ($attrs= $ldap->fetch()){
			$list[$attrs['cn'][0]]=
				utf8_decode($attrs['description'][0]).
				" [".$attrs['cn'][0]."]";
		 }

		/* Show dialog */
		require_once ("edit_lists.inc");
		return;
	}


	/* Show main page */
	require ("edit.inc");
  }

  function remove_from_parent()
  {
	plugin::remove_from_parent();

        /* Zero out arrays */
	foreach (array("goFaxRBlocklist", "goFaxRBlockgroups", "goFaxSBlocklist",
		"goFaxSBlockgroups", "facsimileAlternateTelephoneNumber") as $val){

	        $this->attrs[$val]= array();
	}

	$ldap= $this->config->get_ldap_link();
	$ldap->cd($this->dn);
	$ldap->modify($this->attrs);
	show_ldap_error($ldap->error);

	/* Optionally execute a command after we're done */
	$this->postremove();
  }


  /* Check formular input */
  function check()
  {
	/* Reset message array */
	$message= array();

	/* must: facsimileTelephoneNumber */
	if ($this->facsimileTelephoneNumber == ""){
		$message[]= _("The required field 'Fax' is not set.");
	}

	if (!is_phone_nr($this->facsimileTelephoneNumber)){
		$message[]= _("Please enter a valid telephone number in the 'Fax' field.");
	}

	return ($message);
  }

  /* Save data to object */
  function save_object()
  {
	if (isset($_POST['faxTab'])){
		plugin::save_object();

		/* Adapt combobox values */
		$tmp= 0;
		if (isset($_POST["fax_to_mail"]) && $_POST["fax_to_mail"] == 1){
			$tmp+= 32;
		}
		if (isset($_POST["fax_to_printer"]) && $_POST["fax_to_printer"] == 1){
			$tmp+= 64;
		}
		if (chkacl ($this->acl, "goFaxIsEnabled") == ""){
			if (isset($_POST["goFaxIsEnabled"]) && $_POST["goFaxIsEnabled"] == "1"){
				$this->goFaxIsEnabled= "0";
			} else {
				$this->goFaxIsEnabled= "1";
			}
		}

		if (isset($_POST['facsimileTelephoneNumber'])){
			$this->facsimileTelephoneNumber= $_POST['facsimileTelephoneNumber'];
		}

		/* Write to object */
		if (chkacl ($this->acl, "goFaxDeliveryMode") == ""){
			$this->goFaxDeliveryMode= "$tmp";
		}
	}
  }


  /* Save to LDAP */
  function save()
  {
	plugin::save();

	/* Save arrays */
	foreach (array("goFaxRBlocklist", "goFaxRBlockgroups", "goFaxSBlocklist",
		"goFaxSBlockgroups", "facsimileAlternateTelephoneNumber") as $val){

		$this->attrs[$val]= $this->$val;
	}

	/* Write back to ldap */
	$ldap= $this->config->get_ldap_link();
	$ldap->cd($this->dn);
	$ldap->modify($this->attrs);
	show_ldap_error($ldap->error);

	/* Optionally execute a command after we're done */
	$this->postcreate();
  }


  /* Adapt from template, using 'dn' */
  function adapt_from_template($dn)
  {
	plugin::adapt_from_template($dn);

	foreach (array("goFaxRBlocklist", "goFaxRBlockgroups", "goFaxSBlocklist",
		"goFaxSBlockgroups", "facsimileAlternateTelephoneNumber") as $val){

		if (isset($this->attrs[$val])){
			$this->$val= $this->attrs[$val];
		}
	}
  }



  /* Add alternate fax recipient */
  function addAlternate($number)
  {
	$this->facsimileAlternateTelephoneNumber[]= "$number";
	$this->facsimileAlternateTelephoneNumber=
			array_unique ($this->facsimileAlternateTelephoneNumber);

	sort ($this->facsimileAlternateTelephoneNumber);
	reset ($this->facsimileAlternateTelephoneNumber);
  }

  function delAlternate($numbers)
  {
	$this->facsimileAlternateTelephoneNumber= array_strip ($numbers,
						$this->facsimileAlternateTelephoneNumber);
  }


}

?>
