<?php
/* Load sieve support */
@require_once ("sieve-php.inc");

/*! \brief   mail plugin
    \author  Cajus Pollmeier <pollmeier@gonicus.de>
    \version 2.00
    \date    24.07.2003

    This class provides the functionality to read and write all attributes
    relevant for gosaMailAccounts from/to the LDAP. It does syntax checking
    and displays the formulars required.
 */

class mailAccount extends plugin
{
  /* plugin specific values */
  var $mail= "";
  var $uid= "";
  var $gosaMailAlternateAddress= array();
  var $gosaMailForwardingAddress= array();
  var $gosaMailDeliveryMode= "[L        ]";
  var $gosaMailServer= "";
  var $gosaMailQuota= "";
  var $gosaMailMaxSize= "";
  var $gosaVacationMessage= "";
  var $gosaSpamSortLevel= "";
  var $gosaSpamMailbox= "";

  var $quotaUsage= 0;
  var $forward_dialog= FALSE;
  var $folder_prefix= "";
  var $mailboxList= array();
  var $default_permissions= "none";
  var $member_permissions= "post";
  var $members= array();
  var $admins= array();
  var $perms= array( "lrs" => "read", "lrsp" => "post", "lrsip" => "append",
			"lrswipcd" => "write", "lrswipcda" => "all" );

  /* attribute list for save action */
  var $attributes= array("mail", "gosaMailServer", "gosaMailQuota", "gosaMailMaxSize",
			"gosaMailDeliveryMode", "gosaSpamSortLevel", "gosaSpamMailbox",
			"gosaVacationMessage", "uid");
  var $objectclasses= array("gosaMailAccount");


  /* constructor, if 'dn' is set, the node loads the given
     'dn' from LDAP */
  function mailAccount ($config, $dn= NULL, $shared_folder= FALSE)
  {
        /* Configuration is fine, allways */
        $this->config= $config;

        /* Load bases attributes */
        plugin::plugin($config, $dn);

	if ($dn != NULL){

		/* Load attributes containing arrays */
		foreach (array("gosaMailAlternateAddress", "gosaMailForwardingAddress") as $val){
			if (isset($this->attrs["$val"]["count"])){
				for ($i= 0; $i<$this->attrs["$val"]["count"]; $i++){
					array_push($this->$val, utf8_decode($this->attrs["$val"][$i]));
				}
			}
		}

		/* Preset folder prefix. If we use a shared folder, the prefix 
		   will stay empty, else a "user." is prepended. */
		$this->folder_prefix= ($shared_folder==FALSE)?"user.":"";

		/* Only do IMAP actions if gosaMailServer attribute is set */
		if (isset ($this->attrs["gosaMailServer"][0])){
			$this->getImapAttributes();
		}

	}
  }


  function getImapAttributes()
  {
	/* Connect to IMAP server. I don't want to see these warning here... */
	error_reporting (0);
	$cfg= $this->config->data['SERVERS']['IMAP'][$this->gosaMailServer];
	$this->mailboxList[]= "INBOX";
	$mbox = imap_open($cfg['connect'], $cfg['admin'], $cfg['password'], OP_HALFOPEN);
	error_reporting (E_ALL);

	/* Mailbox reachable? */
	if ($mbox === FALSE){
		print_red (_("Warning: IMAP Server cannot be reached!"));
	} else {

		/* Load quota settings */
		error_reporting (0);
		$quota_value = imap_get_quota($mbox, $this->folder_prefix.$this->uid);
		if(is_array($quota_value)) {
			if (isset($quota_value["STORAGE"]) && is_array($quota_value["STORAGE"])){
				# use for PHP >= 4.3
				$this->quotaUsage= $quota_value["STORAGE"]['usage'];
				$this->gosaMailQuota=  $quota_value["STORAGE"]['limit'];
			} else {
				# backward compatible
				$this->quotaUsage= $quota_value['usage'];
				$this->gosaMailQuota=  $quota_value['limit'];
			}
		}
		error_reporting (E_ALL);

		/* Get list of mailboxes for combo box */
		$list = imap_listmailbox($mbox, $cfg["connect"],
					$this->folder_prefix.$this->uid.".*");
		if (is_array($list)){

			/* Shared folder? */
			if ($this->folder_prefix == ""){
				foreach ($list as $val){
					$this->mailboxList[]=preg_replace ("/.*".$this->uid."\./",
								"INBOX.", imap_utf7_decode ($val));
				}
			/* Normal user account */
			} else {
				foreach ($list as $val){
					$this->mailboxList[]=preg_replace ("/.*user\.".$this->uid."\./",
								"INBOX.", imap_utf7_decode ($val));
				}
			}
		}

		/* imap_getacl available? */
		if (!function_exists('imap_getacl')){
			print_red (_("Warning: imap_getacl is not available, can't get imap permissions!"));
		}

		/* Get permissions in case of shared folders */
		elseif ($this->folder_prefix == ""){
			$this->admins= array();

			$users= imap_getacl ($mbox, $this->uid);

			/* load admin permissions, these are all users with "lrswipcda" rights */
			foreach ($users as $userid => $perms){
				if ($perms == "lrswipcda"){
					$this->admins[]= preg_replace("/^user\./", "", $userid);
				}
			}

			/* load anyone permissions, this are permissions for user "anyone" */
			if (isset ($users['anyone'])){
				if (isset($this->perms[$users['anyone']])){
					$this->default_permissions= $this->perms[$users['anyone']];
				} else {
					$this->default_permissions= "none";
				}
			}

			/* Get member permissions. This is somewhat fuzzy for now because I've
			   only three groups (anyone, member, admin). So I'm looking for the first
			   non admin/anyone and use these permissions for the members. */
			foreach ($users as $userid => $perms){
				if ($userid != "anyone" && $perms != "lrswipcda"){
					if (isset($this->perms[$perms])){
						$this->member_permissions= $this->perms[$perms];
					} else {
						$this->member_permissions= "none";
					}

					break;
				}
			}
		} /* get folder permissions */

		/* Close mbox */
		imap_close ($mbox);

	} /* Mailbox reacable */
  }


  function setImapAttributes()
  {
	error_reporting (0);
	$cfg= $this->config->data['SERVERS']['IMAP'][$this->gosaMailServer];
	$mbox = imap_open($cfg["connect"], $cfg["admin"], $cfg["password"], OP_HALFOPEN);
	error_reporting (E_ALL);

	if ($mbox === FALSE){
		print_red (_("The IMAP server cannot be reached. Note that some changes to mail ".
				"settings are lost.  Please check the changes later on and save ".
				"the user settings again in order to let GOsa write the changes."));
		return (FALSE);
	} else {

		/* Check if mailbox exists */
		$list = imap_listmailbox($mbox, $cfg["connect"],
		$this->folder_prefix.$this->uid);
		if ($list === FALSE){
			if (!imap_createmailbox($mbox, $cfg["connect"].
				$this->folder_prefix.$this->uid)){

				printe ("class mailAccount -> create mailbox", imap_last_error());
			}
		}

		/* Workaround for the php imap extension */
		if ($this->gosaMailQuota == ""){
			$this->gosaMailQuota= "0";
		}

		/* Write mail quota */
		if (!imap_set_quota($mbox, $this->folder_prefix.$this->uid, $this->gosaMailQuota)){
			printe ("class mailAccount -> setMailQuota", imap_last_error());
		}

		/* Set shared folder acl's */
		if ($this->folder_prefix == ""){

			/* Remove all acl's */
			$users= imap_getacl ($mbox, $this->uid);
			foreach ($users as $userid => $perms){
				imap_setacl ($mbox, $this->uid, $userid, "");
			}

			/* Set new acl's */
			foreach (array("default_permissions", "member_permissions") as $val){
				if (!isset($this->perms[$this->$val])){
					$this->$val= "";
				} else {
					$this->$val= $this->perms[$this->$val];
				}
			}
			imap_setacl ($mbox, $this->uid, "anyone", $this->default_permissions);
			foreach ($this->members as $user){
				imap_setacl ($mbox, $this->uid, "user.".$user, $this->member_permissions);
			}
			foreach ($this->admins as $user){
				imap_setacl ($mbox, $this->uid, "user.".$user, "lrswipcda");
			}
		}
	
		/* Shutdown IMAP connection */
		imap_close($mbox);
	}
	return (TRUE);
  }


  function setSieveAttributes()
  {
	$cfg= $this->config->data['SERVERS']['IMAP'][$this->gosaMailServer];	

	/* Build spamlevel. Spamassassin tags mails with "*" for each integer
	   point of spam. So a spam level of 5.3 gets "*****" which can be
	   checked easily by spam filters */
	$spamlevel= "";
	for ($i= 0; $i<$this->gosaSpamSortLevel; $i++){
		$spamlevel .= "*";
	}

	/* Log into the mail server */
	$sieve=new sieve($cfg["sieve_server"], $cfg["sieve_port"], $this->uid,
			$cfg["password"], $cfg["admin"]);

	if (!$sieve->sieve_login()){
			printe ("class mailAccount -> sieve login", to_string($sieve->error_raw));
	}

	/* Load current script from server and remove everything between the comments
	   "###GOSA" */
	$script= "";
	if($sieve->sieve_listscripts()){
		if (in_array("gosa", $sieve->response)){

			/* get old GOsa script */
			if(!$sieve->sieve_getscript("gosa")){
				printe ("class mailAccount -> sieve getscript", to_string($sieve->error_raw));
			}

			foreach ($sieve->response as $line){
				if (preg_match ("/^###GOSA/", $line)){
					break;
				}
				$line= rtrim($line);
				if (!preg_match ('/^\s*$/', $line)){
					$script .= $line."\n";
				}
			}

		}
	}


#### FIXME
# This would be fine for reading in parts of the sieve script
# from files
#$text= implode ("", file($fname));
#eval ("\$text=\"$text\";");
####

	/* Only create a new one, if it is not empty */
	if (is_integer(strpos($this->gosaMailDeliveryMode, "R")) ||
		is_integer(strpos($this->gosaMailDeliveryMode, "C")) ||
		!is_integer(strpos($this->gosaMailDeliveryMode, "L")) ||
		is_integer(strpos($this->gosaMailDeliveryMode, "V")) ||
		is_integer(strpos($this->gosaMailDeliveryMode, "S"))){

		$script .= <<<EOD
###GOSA
require ["fileinto", "reject", "vacation"];

EOD;
	}

	/* Add anti-spam code */
	if (is_integer(strpos($this->gosaMailDeliveryMode, "S"))){
		$script .= <<<EOD
# Sort mails with higher spam level
if header :contains "X-Spam-Level" "$spamlevel" {
  fileinto "$this->gosaSpamMailbox";
}

EOD;
	}

	/* Add "reject due to mailsize" code, message is currently not
	   adjustable through GOsa. */
	if (is_integer(strpos($this->gosaMailDeliveryMode, "R"))){
		$script .= <<<EOD
#  Reject mails with bigger size
if size :over {$this->gosaMailMaxSize}M{
        reject text:
Dear sender,

the mail you sent to our mailsystem has been rejected due
to a user configured maximum mail size ($this->gosaMailMaxSize MB).

Either ask the user to remove the sizelimit, or send smaller pieces.

Thank you,
 the mailserver
.
        ;
        discard;
}

EOD;
	}

	/* Add vacation information */
	if (is_integer(strpos($this->gosaMailDeliveryMode, "V"))){

		/* Sieve wants all destination addresses for the
		   vacation message, so we've to assemble them from
		   mail and mailAlternateAddress */
		$addrlist= "\"".$this->mail."\"";
		foreach ($this->gosaMailAlternateAddress as $val){
			$addrlist .= ", \"$val\"";
		}
		$script .= <<<EOD
# Vacation message
vacation :addresses [$addrlist]
"$this->gosaVacationMessage" ;

EOD;
	}

	/* If no local delivery is wanted, tell the script to discard the mail */
	if (!is_integer(strpos($this->gosaMailDeliveryMode, "L"))){
		$script .= <<<EOD
# Do not deliver to own mailbox
discard;

EOD;
	}

	/* Just be aware of null scripts... */
	if (!isset ($script)){
		$script= "";
	}

	/* Upload script and make it the default one */
	if (!$sieve->sieve_sendscript("gosa", $script)){
		printe ("class mailAccount -> send sieve script", to_string($sieve->error_raw));
	}
	if(!$sieve->sieve_setactivescript("gosa")){
		printe ("class mailAccount -> set active script", to_string($sieve->error_raw));
	}

	$sieve->sieve_logout();
  }


  function execute()
  {
	/* Do we need to flip is_account state? */
	if (isset($_POST['modify_state'])){
		$this->is_account= !$this->is_account;
	}

	/* Do we represent a valid account? */
	if (!$this->is_account && $this->parent == NULL){
		echo "<img src=\"images/stop.png\" align=center>&nbsp;<b>".
		_("This 'dn' has no valid mail extensions.")."</b>";
		return;
	}

	/* Show tab dialog headers */
	if ($this->parent != NULL){
		if ($this->is_account){
			$this->show_header(_("Remove mail account"),
				_("This account has mail features enabled. ".
				  "You can disable them by clicking below."));
		} else {
			$this->show_header(_("Create mail account"),
				_("This account has mail features disabled. ".
				  "You can enable them by clicking below."));
			return;
		}
	}

	/* Check sorting variable */
	$this->check_sortranges();

	/* Trigger forward add dialog? */
	if (isset($_POST['add_local_forwarder'])){
		$this->forward_dialog= TRUE;
		$this->dialog= TRUE;
	}

        /* Cancel forward add dialog? */
        if (isset($_POST['add_locals_cancel'])){
                $this->forward_dialog= FALSE;
                $this->dialog= FALSE;
        }

	/* Finished adding of locals? */
	if (isset($_POST['add_locals_finish'])){
		if (count ($_POST['local_list']) &&
			chkacl ($this->acl, "gosaMailForwardingAddress") == ""){

			/* Walk through list of forwarders, ignore own addresses */
			foreach ($_POST['local_list'] as $val){
				if (!in_array ($val, $this->gosaMailAlternateAddress) &&
					$val != $this->mail){

					$this->addForwarder($val);
				}
			}
		}
		$this->forward_dialog= FALSE;
		$this->dialog= FALSE;
	}

	/* Add forward email addresses */
	if (isset($_POST['add_forwarder'])){
		if ($_POST['forward_address'] != ""){

			/* Valid email address specified? */
			$address= $_POST['forward_address'];
			if (!is_email($address)){

				print_red (_("You're trying to add an invalid email address ".
					"to the list of forwarders."));

			} elseif ($address == $this->mail
				|| in_array($address, $this->gosaMailAlternateAddress)) {

				print_red (_("Adding your one of your own addresses to the ".
					"forwarders makes no sense."));

			} else {

				/* Add it */
				if (chkacl ($this->acl, "gosaMailForwardingAddress") == ""){
					$this->addForwarder ($address);
				}

			}
		}
	}

	/* Delete forward email addresses */
	if (isset($_POST['delete_forwarder'])){
		if (count($_POST['forwarder_list']) 
			&& chkacl ($this->acl, "gosaMailForwardingAddress") == ""){

			$this->delForwarder ($_POST['forwarder_list']);
		}
	}

	/* Add alternate email addresses */
	if (isset($_POST['add_alternate'])){
		if ($_POST['alternate_address'] != "" &&
			chkacl ($this->acl, "gosaMailAlternateAddress") == ""){

			if (!is_email($_POST['alternate_address'])){
				print_red (_("You're trying to add an invalid email address ".
						"to the list of alternate addresses."));

			} elseif (($user= $this->addAlternate ($_POST['alternate_address'])) != ""){
				if ($user != $ui->username){
					print_red (_("The address you're trying to add is already".
						" used by user")." '$user'.");
				}
			}
		}
	}

	/* Delete alternate email addresses */
	if (isset($_POST['delete_alternate']) && isset ($_POST['alternates_list'])){
		if (count($_POST['alternates_list']) &&
			chkacl ($this->acl, "gosaMailAlternateAddress") == ""){

			$this->delAlternate ($_POST['alternates_list']);
		}
	}

	/* Show forward add dialog */
        if ($this->forward_dialog){
		$ldap= $this->config->get_ldap_link();

		/* Get actual list */
		$mailusers= array ();
		if (isset ($_POST['department'])){
			$ldap->cd ($this->config->departments[$_POST['department']]);
		} else {
			$ldap->cd ($this->config->current['BASE']);
		}
		$ldap->search ("(objectClass=gosaMailAccount)");
		error_reporting (0);
		while ($attrs= $ldap->fetch()){
			if (preg_match ($this->sortranges[$this->sorting], $attrs["sn"][0])){
				$mailusers[$attrs['mail'][0]]= utf8_decode($attrs['sn'][0]).", ".
						utf8_decode($attrs['givenName'][0])." &lt;".
						$attrs['mail'][0]."&gt;";
			}
		}
		error_reporting (E_ALL);
		natcasesort ($mailusers);
		reset ($mailusers);

		/* Show dialog */
		if ($this->folder_prefix == ""){
			require_once ("mailgroup_locals.inc");
		} else {
			require_once ("edit_locals.inc");
		}

                return;
        }

	/* Show main page */
	if ($this->folder_prefix == ""){
		require ("mailgroup.inc");
	} else {
		require ("edit.inc");
	}
  }


  /* remove object from parent */
  function remove_from_parent()
  {
        /* include global link_info */
        $ldap= $this->config->get_ldap_link();

        /* Remove and write to LDAP */
        plugin::remove_from_parent();

	/* Zero arrays */
	$this->attrs['gosaMailAlternateAddress']= array();
	$this->attrs['gosaMailForwardingAddress']= array();

	/* Keep uid */
	unset ($this->attrs['uid']);
        @DEBUG (DEBUG_LDAP, __LINE__, __FUNCTION__, __FILE__,
                $this->attributes, "Save");
	$ldap->cd($this->dn);
        $ldap->modify($this->attrs);
	show_ldap_error($ldap->error);

	/* Optionally execute a command after we're done */
	$this->postremove();
  }


  /* Save data to object */
  function save_object()
  {
	if (isset($_POST['mailTab'])){
		/* Save ldap attributes */
		plugin::save_object();

		/* Assemble mail delivery mode
		   The mode field in ldap consists of values between braces, this must
		   be called when 'mail' is set, because checkboxes may not be set when
		   we're in some other dialog.

		   Example for gosaMailDeliveryMode [LR        ]
		   L: Local delivery
		   R: Reject when exceeding mailsize limit
		   S: Use spam filter
		   V: Use vacation message
		   C: Use custm sieve script
		   I: Only insider delivery */

		$tmp= "";
		if (!isset($_POST["drop_own_mails"])){
			$tmp.= "L";
		}
		if (isset($_POST["use_mailsize_limit"])){
			$tmp.= "R";
		}
		if (isset($_POST["use_spam_filter"])){
			$tmp.= "S";
		}
		if (isset($_POST["use_vacation"])){
			$tmp.= "V";
		}
		if (isset($_POST["own_script"])){
			$tmp.= "C";
		}
		if (isset($_POST["only_local"])){
			$tmp.= "I";
		}
		$tmp= "[$tmp]";

		if (chkacl ($this->acl, "gosaMailDeliveryMode") == ""){
			$this->gosaMailDeliveryMode= $tmp;
		}
	}
  }


  /* Save data to LDAP, depending on is_account we save or delete */
  function save()
  {
	$ldap= $this->config->get_ldap_link();

	/* Call parents save to prepare $this->attrs */
	plugin::save();

	/* Save arrays */
	$this->attrs['gosaMailAlternateAddress']= $this->gosaMailAlternateAddress;
	$this->attrs['gosaMailForwardingAddress']= $this->gosaMailForwardingAddress;

	/* Save data to LDAP */
	$ldap->cd($this->dn);
	$ldap->modify($this->attrs);
	show_ldap_error($ldap->error);

	/* Only do IMAP actions if we are not a template */
	if (!$this->is_template){
		if ($this->setImapAttributes()){
		
			/* Write sieve information only if not in C or shared folder mode */
			if (!is_integer(strpos($this->gosaMailDeliveryMode, "C"))
				&& !$this->folder_prefix == ""){

				$this->setSieveAttributes();
			}
		}
	}

	/* Optionally execute a command after we're done */
	$this->postcreate();
  }

  /* Check formular input */
  function check()
  {
	$ldap= $this->config->get_ldap_link();

	$message= array();

	/* must: mail */
	if ($this->mail == ""){
		$message[]= _("The required field 'Primary address' is not set.");
	}
	if (!is_email($this->mail)){
		$message[]= _("Please enter a valid email addres in 'Primary address' field.");
	}
	$ldap->cd($this->config->current['BASE']);
	$ldap->search ("(&(|(mail=".$this->mail.")(gosaMailAlternateAddress=".
			$this->mail."))(!(uid=".$this->uid."))(!(cn=".$this->uid.")))");
	if ($ldap->count() != 0){
		$message[]= _("The primary address you've entered is already in use.");
	}

	/* Check quota */
	if ($this->gosaMailQuota != '' && chkacl ($this->acl, "gosaMailQuota") == ""){
		if (!is_numeric($this->gosaMailQuota)) {
			$message[]= _("Value in 'Quota size' is not valid.");
		} else {
			$this->gosaMailQuota= (int) $this->gosaMailQuota;
		}
	}

	/* Check rejectsize for integer */
	if ($this->gosaMailMaxSize != '' && chkacl ($this->acl, "gosaMailQuota") == ""){
		if (!is_numeric($this->gosaMailMaxSize)){
			$message[]= _("Please specify a vaild mail size for mails to be rejected.");
		} else {
			$this->gosaMailMaxSize= (int) $this->gosaMailMaxSize;
		}
	}

	/* Need gosaMailMaxSize if use_mailsize_limit is checked */
	if (is_integer(strpos($this->gosaMailDeliveryMode, "R")) && 
		$this->gosaMailMaxSize == ""){

		$message[]= _("You need to set the maximum mail size in order to reject anything.");
	}

	return ($message);
  }

  /* Adapt from template, using 'dn' */
  function adapt_from_template($dn)
  {
	plugin::adapt_from_template($dn);

	foreach (array("gosaMailAlternateAddress", "gosaMailForwardingAddress") as $val){
		$this->$val= array();
		if (isset($this->attrs["$val"]["count"])){
			for ($i= 0; $i<$this->attrs["$val"]["count"]; $i++){
				$value= utf8_decode($this->attrs["$val"][$i]);
				foreach (array("sn", "givenName", "uid") as $repl){
					if (preg_match("/%$repl/i", $value)){
						$value= preg_replace ("/%$repl/i", $this->parent->$repl, $value);
					}
				}
				array_push($this->$val, rewrite($value));
			}
		}
	}
	$this->mail= rewrite($this->mail);
  }

  /* Add entry to forwarder list */
  function addForwarder($address)
  {
	$this->gosaMailForwardingAddress[]= $address;
	$this->gosaMailForwardingAddress= array_unique ($this->gosaMailForwardingAddress);

	sort ($this->gosaMailForwardingAddress);
	reset ($this->gosaMailForwardingAddress);
  }

  /* Remove list of addresses from forwarder list */
  function delForwarder($addresses)
  {
	$this->gosaMailForwardingAddress= array_strip ($addresses, $this->gosaMailForwardingAddress);
  }



  function addAlternate($address)
  {
	$ldap= $this->config->get_ldap_link();

	$address= strtolower($address);

	/* Is this address already assigned in LDAP? */
	$ldap->cd ($this->config->current['BASE']);
	$ldap->search ("(&(objectClass=gosaMailAccount)(|(mail=$address)".
				"(gosaMailAlternateAddress=$address)))");

	if ($ldap->count() > 0){
		$attrs= $ldap->fetch ();
		return ($attrs["uid"][0]);
	}

	/* Add to list of alternates */
	if (!in_array($address, $this->gosaMailAlternateAddress)){
		$this->gosaMailAlternateAddress[]= $address;
	}

	sort ($this->gosaMailAlternateAddress);
	reset ($this->gosaMailAlternateAddress);

	return ("");
  }


  function delAlternate($addresses)
  {
	$this->gosaMailAlternateAddress= array_strip ($addresses, $this->gosaMailAlternateAddress);
  }

}

?>
